# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 19:23:42 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

create():                   Creates an image.
delete():
save():                     Save image.
load():                     Load image from .pcf.
get_amount():
get_size():                 Get image size and parameter.
rotate():                   Rotates image to desired image.
show():                     Show image.

"""
import sys
import os
import datetime
from skimage import io
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import change_this.roots as root
import variables.dicts as dic
import technoteam.activex as ax

def create(image=0, name='Evaluation[1]'):
    """
    Creates an image.|
    ---------------------------
    The image gets the size of the previous image (luminance image or previous
    evalutation image.)
    ---------------------------------------------------------------------------

    %timeit:
        41.4 ms ± 41.5 ms per loop (mean ± std. dev. of 7 runs, 1 loop each)

    Parameters
    ----------
    LMK :
        Dispatch('lmk4.LMKAxServer')
    image : int, optional
        0=Grey image, 1=Color image. The default is 0.
    name : QString, optional
        Wished image  name. The default is 'Evaluation[1]'.

    Returns
    -------
    index : int
        Index of the image created

    """

    err_code, index = ax.LMK.iImageCreate(image, name)
    ax.error_code(err_code)

    return index

def delete(image=dic.IMAGE_TYPES['Color']):
    """
    [ADD THIS]

    %timeit:
        27.2 ms ± 373 µs per loop (mean ± std. dev. of 7 runs, 10 loops each)

    Parameters
    ----------
    image : TYPE, optional
        DESCRIPTION. The default is dic.IMAGE_TYPES['Color'].

    Returns
    -------
    None.

    """
    ax.LMK.iImageDelete(image)

def save(image=dic.IMAGE_TYPES['Color'], file_name=root.SAVE + \
         datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S') + \
             dic.FILE_TYPES['png']):
    """
    Save image.|
    -----------
    The function overwrites an existing file.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default -1)
            | Index of image to save
        :file_name: QString (default: 'C:/Desktop/Image.png')
            | Destination file name
            | Datetime is the exact datetime of the measurement, not datetime when saved.
    """
    if not os.path.exists(root.SAVE):
        os.makedirs(root.SAVE)
    err_code = ax.LMK.iSaveImage(image, file_name)
    ax.error_code(err_code) # Check for error

def load(image=dic.IMAGE_TYPES['Color'], file_name=root.LOAD + 'Image' + dic.FILE_TYPES['pcf']):
    """
    Load image from .pcf.|
    ---------------------
    Loads a saved image from .pcf. Of course, the image needs to be saved
    first.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default -1)
            | Index of image to Load to
        :file_name: QString (default: MEAS_ROOT + 'Image.pcf')
            | Source file name
            | .pus = Camera Image
            | .pf = Luminance Image
            | .pcf = Color Image
    """
    err_code = ax.LMK.iLoadImage(image, file_name)
    ax.error_code(err_code) # Check for error

def get_amount():
    """
    [ADD THIS]

    %timeit:
        127 µs ± 2.64 µs per loop (mean ± std. dev. of 7 runs, 10000 loops each)

    Returns
    -------
    images_no : TYPE
        DESCRIPTION.

    """

    err_code, images_no = ax.LMK.iGetNumberImages()
    ax.error_code(err_code) # Check for error

    return images_no

def get_size(image=dic.IMAGE_TYPES['Color']):
    """
    Get image size and parameter.|
    -----------------------------
    These informations are needed for the other access functions.
    See also Image.SetSize()
    -----------------------------------------------------------------------

    %timeit:
        142 µs ± 6.26 µs per loop (mean ± std. dev. of 7 runs, 10000 loops each)

    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: -1)
            | Index of image to inform about
    Returns:
        | Number of lines is LastLine - FirstLine + 1.
        | Number of columns is LastColumn - FirstColumn + 1.
        :first_line: int
            | Gets the index of the first line
        :last_line: int
            | Gets the index of the last line
        :first_col: int
            | Gets the index of the first column
        :last_col: int
            | Gets the index of the last column
        :dimensions: int
            | 1 = gray images
            | 3 = color images
    """
    err_code, image_first_line, image_last_line, image_first_col, \
        image_las_col, image_dimensions = ax.LMK.iImageGetSize(image)
    ax.error_code(err_code) # Check for error

    return image_first_line, image_last_line, image_first_col, \
        image_las_col, image_dimensions

def rotate(code=dic.OPERATION_TYPES['Rotate'],
           src_image=dic.IMAGE_TYPES['Luminance'], param='180',
           dst_image=dic.IMAGE_TYPES['Evaluation[1]']):
    """

    Rotates image to desired image.|
    -------------------------------

    Uses image arithmetic with one source image, one parameter, and one
    desination image.
    ---------------------------------------------------------------------------

    %timeit:
        149 ms ± 10.4 ms per loop (mean ± std. dev. of 7 runs, 10 loops each)

    Parameters
    ----------
    LMK :
        Dispatch('lmk4.LMKAxServer')
    code : QString, optional
        Operation code, see OPERATION_TYPES{}. The default is OPERATION_TYPES['Rotate'].
    src_image : int, optional
        Index of source image. The default is IMAGE_TYPES['Luminance'].
    param : QString, optional
        Parameter of algorithm. The default is '180'.
    dst_image : int, optional
        Index of destination image. The default is IMAGE_TYPES['Evaluation[1]'].

    Returns
    -------
    None.

    """
    err_code = ax.LMK.imageArithmeticIP1(code, src_image, param, dst_image)
    ax.error_code(err_code) # Check for error

def show(file_name=root.LOAD + 'Image' + dic.FILE_TYPES['png']):
    """
    Show image.|
    -----------
    Shows a saved image. Of course, the image needs to be saved first.
    -----------------------------------------------------------------------
    Parameters:
        :file_name: QString (default: MEAS_ROOT + 'Image.png')
            | Source file name
    Returns:
        :image: uint8
            | Stores image into a numpy array and show it
    """
    image = io.imread(file_name)
    io.imshow(image)

    return image



# -*- coding: utf-8 -*-
"""
Created on Wed Feb 12 09:52:14 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Change these roots to your own.

"""
import datetime

# Define Calibration Data Root
DATA = 'F:/LMK/Calibration Data'

# Define Save Root
SAVE = 'E:/Measurements/' + str(datetime.date.today()) + '/'
SAVE_TEST = 'E:/Measurements/TEST/'

# Define Load Root
LOAD = 'E:/Measurements/2020-03-05/'
LOAD_HYP = 'F:/Program Files/Hyperspectral Camera/GUIHSK23052016/Data_Cube/20200305_081505/'



# -*- coding: utf-8 -*-
"""
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

VirtualRealityHmd():        Measure a Virtual Reality Head-Mounted-Display.
    __init__():             Initializes LMK for HMD
    warm_up():              Warm-Up VR-HMD
    measure():              Measure VR-HMD
    analyze():              Analyze Data from Measurement

"""
import os
import datetime
import time
import numpy as np

from technopy.change_this import roots as root
from technopy.variables import dicts as dic
from technopy.useful import bob_the_builder as bob
from technopy.technoteam import labsoft as ls
from technopy.technoteam import camera as cam
from technopy.technoteam import capture as cap
from technopy.technoteam import image as im
from technopy.technoteam import region as reg
from technopy.technoteam import evaluation as eva
from technopy.useful import fantasia as fanta


class VirtualRealityHmd:
    """
    [ADD THIS]
    """

    def __init__(self, camera='VR', lens='Conoscopic', mod_freq=90.0,
                 connect_camera=True):
        """
        Initializes LMK for VR-HMD.|
        ---------------------------
        %timeit:
            2.28 s ± 56.5 ms per loop (mean ± std. dev. of 7 runs, 1 loop each)

        Parameters
        ----------
        camera : TYPE, optional
            DESCRIPTION. The default is 'VR'.
        lens : TYPE, optional
            DESCRIPTION. The default is 'Conoscopic'.
        mod_freq : TYPE, optional
            DESCRIPTION. The default is 90.0.
        autoscan : TYPE, optional
            DESCRIPTION. The default is False.
        min_time : TYPE, optional
            DESCRIPTION. The default is 0.0.
        time_ratio : TYPE, optional
            DESCRIPTION. The default is 3.0.
        pic_count : TYPE, optional
            DESCRIPTION. The default is 1.
        connect_camera : TYPE, optional
            DESCRIPTION. The default is True.

        Returns
        -------
        None.

        """

        self.save_root = root.SAVE
        self.load_root = root.LOAD
        self.camera = camera
        self.lens = lens
        self.mod_freq = mod_freq

        ## Initialize
        # Open LMK LabSoft4 Standard Color ActiveX
        ls.open_labsoft()
        if connect_camera is True:
            _, self.camera_no = cam.set_camera(self.camera)
            _, self.lens_no = cam.set_lens(self.camera, self.lens)
            cam.open_camera(self.camera_no, self.lens_no)

            ## Adjust Camera
            # Set Modulation Frequency
            cam.set_modulation_frequency(self.mod_freq)
            # Change converting units so it doesn't multiply by two
            cam.set_converting_units()

    def warm_up(self, time_total=120):
        """
        [ADD THIS]

        Parameters
        ----------
        time_total : TYPE, optional
            DESCRIPTION. The default is 120.

        Returns
        -------
        output : TYPE
            DESCRIPTION.

        """

        time_warm = 0
        output = []

        while time_warm < time_total:
            start = time.time()
            if datetime.datetime.now().minute % 1 == 0:
                max_lum = bob.max_luminance()
                ls.close_labsoft()
                print(time_warm + 1, '(of', time_total, 'minutes)', '|',
                      datetime.datetime.now().time(),
                      '|', max_lum, 'cd/m^2 (max)')
            time_warm = time_warm + 1
            output.append([time_warm, max_lum])
            overlay = time.time() - start
            time.sleep(60 - overlay)

        if output != []:
            output = np.vstack((output))

        return output


    def measure(self, autoscan=False, min_time=0.0, time_ratio=3.0,
                pic_count=1):
        """
        Measure a Virtual Reality Head-Mounted-Display.|
        -----------------------------------------------

        Returns
        -------
        output_color : TYPE
            DESCRIPTION.

        """
        meas_start = datetime.datetime.now()

        ## Capture Image
        cap.color_high_dyn(autoscan, min_time, time_ratio, pic_count)

        ## Save Measurement
        ls.save(file_name=self.save_root + \
                datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S') + \
                    dic.FILE_TYPES['ttcs'])

        meas_fin = datetime.datetime.now()
        print('Measured Image in: {}\n'.format(meas_fin - meas_start))

    def analyze(self, target='XYZ'):
        """
        [ADD THIS]

        %timeit:
            target='Y':
            36.2 s ± 1.55 s per loop (mean ± std. dev. of 7 runs, 1 loop each)
            target='XYZ':
            36.8 s ± 2.06 s per loop (mean ± std. dev. of 7 runs, 1 loop each)

        Parameters
        ----------
        target : TYPE, optional
            DESCRIPTION. The default is 'XYZ'.

        Returns
        -------
        results : TYPE
            DESCRIPTION.

        """

        results = []

        for measurement in os.listdir(self.load_root):
            if measurement.endswith('.ttcs'):

                ls.load(self.load_root + measurement)

                if target == 'Y':
                    color = 0
                    image_name = 'Evaluation[1]'
                    statistic = dic.STATISTIC_TYPES['standardGrey']

                elif target == 'XYZ':
                    color = 1
                    image_name = 'Evaluation[2]'
                    statistic = dic.STATISTIC_TYPES['standardColor']

                region_name = '1'
                images_no = im.get_amount()
                if images_no != 0:
                    im.delete()

                image = im.create(color, image_name)

                if target == 'Y':
                    eva.coord_trans_lum()
                elif target == 'XYZ':
                    eva.coord_trans_col()

                # If region already exists, delete it
                err_code, index_out = \
                    reg.get_id(dic.IMAGE_TYPES[image_name],
                               region_name)
                if err_code == 0:
                    reg.delete(dic.IMAGE_TYPES[image_name],
                               index_out)

                # If statistic already exists, delete it
                exists, _, _ = eva.statistic_exists(dic.IMAGE_TYPES[image_name],
                                                    index_out)
                if exists == 1:
                    eva.delete_statistic()

                reg.create_rect(image)

                # Get ID of region
                _, index_out = reg.get_id(image,
                                          region_name)
                # Select region from index of region
                reg.select(image, index_out)

                ## Evaluate Region
                eva.create_statistic(statistic,
                                     image,
                                     index_out)

                if target == 'Y':
                    output = eva.get_max_lum()

                elif target == 'XYZ':
                    output = eva.get_xyz(index_out)

                results.append(output)

        if results != []:
            results = np.vstack((results))

        return results

if __name__ == '__main__':

    VR = VirtualRealityHmd(connect_camera=False)

    # label = 'Oculus Rift CV1'

    # MAX_Ys = VR.analyze('Y')
    # XYZs = VR.analyze('XYZ')

    # diff, mean = fanta.y_diff(MAX_Ys, XYZs)
    # u_v_ = fanta.xyz_to_u_v_(XYZs)
    # u_v_ = u_v_.astype(float)
    # fanta.show(space='uv', new=False, color='r', input_type='file',
    #       count=7, output=u_v_, label=label)
    # fanta.uv(u_v_[3,0], u_v_[3,1], label=label, color='r', new=False)



# -*- coding: utf-8 -*-
"""
Created on Sun Feb  9 22:08:49 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

connect():
send():
disconnect():

"""
import smtplib

# Setup an email address to email from and to
# https://automatetheboringstuff.com/chapter16/
def connect(email, password, smtp='smtp.gmail.com', gate=587):
    """
    [ADD THIS]

    Parameters
    ----------
    email : TYPE
        DESCRIPTION.
    password : TYPE
        DESCRIPTION.
    smtp : TYPE, optional
        DESCRIPTION. The default is 'smtp.gmail.com'.
    gate : TYPE, optional
        DESCRIPTION. The default is 587.

    Returns
    -------
    smtp_obj : TYPE
        DESCRIPTION.

    """

    # Connect to an SMTP server
    smtp_obj = smtplib.SMTP(smtp, gate)
    # Establish a connection to the SMTP server
    smtp_obj.ehlo()
    # Enable encryption for connection
    smtp_obj.starttls()
    # Enter login details
    smtp_obj.login(email, password)

    return smtp_obj

# Send email to given email address
def send(smtp_obj, email):
    """
    [ADD THIS]

    Parameters
    ----------
    smtp_obj : TYPE
        DESCRIPTION.
    email : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    # Send mail with text to email
    smtp_obj.sendmail(email, email, 'Subject: Measurement\nMeasurement finished.')

def disconnect(smtp_obj):
    """
    [ADD THIS]

    Parameters
    ----------
    smtp_obj : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    # Close email connection
    smtp_obj.quit()



# -*- coding: utf-8 -*-
"""
Created on Sun Feb  9 22:06:28 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

# get_no_displays():          Number of connected displays.
# get_target():               Get current target for template image generation.
# set_target():               Set target for template image generation.
# get_target_properties():    Get properties of current target.
# get_list_of_categories():   Get list of categories of template images.
# get_list_of_operations():   Get list of tpes of tamplate images for the given category.
# set_operation():            Set a type of template or an option.
# get_list_of_param_names():  Get a parameter name list for the given template image type.
# get_param_value():          Get a certain parameter value.
# set_param_value():          Set a certain parameter value.
# hide_dialog():              Make template image dialog invisible.
# create_image():             Create template image with the adjusted parameters.
# delete_image():             Remove template image from the display.
# load_image():               Load template image file to the target.

"""
import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import technoteam.activex as ax

def get_no_displays():
    """
    Number of connected displays.|
    -----------------------------
    At lease there is one - on which the program is shown.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :no_displays: int
            | Get number of available displays, at least one.
    """
    err_code, no_displays = ax.LMK.iTIG_GetNumberTargetDisplays()
    ax.error_code(err_code) # Check for error

    return no_displays

def get_target():
    """
    Get current target for template image generation.|
    -------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :display_id: int
            | -1 = Use given image name
            | 0...N-1 = One of the connected displays
        :display_name: QString
            | Image name is only returned if 'DisplayID' == -1
    """
    err_code, display_id, display_name = ax.LMK.iTIG_GetTarget()
    ax.error_code(err_code) # Check for error

    return display_id, display_name

def set_target(display_id, display_name):
    """
    Get current target for template image generation.|
    -------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :display_id: int
            | -1 = Use given image name
            | 0...N-1 = One of the connected displays
        :display_name: QString
            | Image name is only necessary if 'DisplayID' == -1
    """
    err_code = ax.LMK.iTIG_SetTarget(display_id, display_name)
    ax.error_code(err_code) # Check for error

def get_target_properties():
    """
    Get properties of current target.|
    ---------------------------------
    The size of the target image is fixed.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :target_type: int
            | 1 = Monochrome image
            | 2 = Color image
            | 3 = Display
            |       in this case the type (monochrome or color)
            |       is chosen automatically
        :target_height: int
            | Height in millimeters
        :target_width: int
            | Width in millimeters
        :target_lines: int
            | Number of image lines
        :target_columns: int
            | Number of image columns
    """
    err_code, target_type, target_height, \
        target_width, target_lines, \
            target_columns = ax.LMK.iTIG_GetTargetProperties()
    ax.error_code(err_code) # Check for error

    return target_type, target_height, target_width, target_lines, target_columns

def get_list_of_categories():
    """
    Get list of categories of template images.|
    ------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :categories: QStringList
            | Shows list of categories for TIG
    """
    err_code, categories = ax.LMK.iTIG_GetListOfCategories()
    ax.error_code(err_code) # Check for error

    return categories

def get_list_of_operations(category):
    """
    Get list of types of template images for the given category.|
    ------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :category: QString
            | Chosen category
    Returns:
        :operations: QStringList
            | Shows list of template image types for given category
    """
    [err_code, operations] = ax.LMK.iTIG_GetListOfOperations(category)
    ax.error_code(err_code) # Check for error

    return operations

def set_operation(category, operation, check=1):
    """
    Set a type of template image or an option.|
    ------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :category: QString
            | Name of category
        :operation: QString
            | Name of operation or option
        :check: int
            | Set or unset.
            | This parameter is only valied if this is an option.
            |   (checkbox, not a radio button)
    """
    err_code = ax.LMK.iTIG_SetOperation(category, operation, check)
    ax.error_code(err_code) # Check for error

def get_list_of_param_names():
    """
    Get a parameter name list for the given template image type.|
    ------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :parameters: QStringList
            | Show parameter names in a list
    """
    err_code, parameters = ax.LMK.iTIG_GetListOfParameterNames()
    ax.error_code(err_code) # Check for error

    return parameters

def get_param_value(parameter):
    """
    Get a certain parameter value.|
    ------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :parameter: QString
            | Name of parameter
    Returns:
        :parameter_value: float
            | Value of parameter (if bool, then 0 or 1)
    """
    err_code, parameter_value = ax.LMK.iTIG_GetParameterValue(parameter)
    ax.error_code(err_code) # Check for error

    return parameter_value

def set_param_value(parameter, parameter_value):
    """
    Set a certain parameter value.|
    ------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :parameter: QString
            | Name of parameter
        :parameter_value: float
            | Value of parameter (if bool, then 0 or 1)
    """
    err_code = ax.LMK.iTIG_SetParameterValue(parameter, parameter_value)
    ax.error_code(err_code) # Check for error

def show_dialog():
    """
    Make template image dialog visible.|
    -----------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    """
    err_code = ax.LMK.iTIG_ShowDialog()
    ax.error_code(err_code) # Check for error

def hide_dialog():
    """
    Make template image dialog invisible.|
    -------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    """
    err_code = ax.LMK.iTIG_HideDialog()
    ax.error_code(err_code) # Check for error

def create_image():
    """
    Create template image with the adjusted parameters.|
    ---------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    """
    err_code = ax.LMK.iTIG_CreateImage()
    ax.error_code(err_code) # Check for error

def delete_image():
    """
    Remove template image from the display.|
    ---------------------------------------
    Template images can only be deleted from a display. If the target is an
    evaluation image, deletion is not possible.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    """
    err_code = ax.LMK.iTIG_DeleteImage()
    ax.error_code(err_code) # Check for error

def load_image(file_name):
    """
    Load template image file to the target.|
    ---------------------------------------
    This operation is only allowed for category 'User Defined'.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :file_name: QString
            | Name of an image file.
            | Allowed file extensions:
            |   .puc, .pus, .pf, .pco, .pcf, .bmp, .png, .jpg
    """
    err_code = ax.LMK.iTIG_LoadImage(file_name)
    ax.error_code(err_code) # Check for error



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 19:19:29 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

single_pic():               SinglePic capture algorithm.
multi_pic():                MultiPic capture algorithm.
high_dyn_pic():             HighDyn capturing for luminance image.
color_high_dyn():           HighDyn capturing for color image.
get_last_info():            Determine information about the preceeding capture.


"""
import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import operator
import datetime

import technoteam.activex as ax
import technoteam.camera as cam

def single_pic(autoscan=True, exposure_time=0.1):
    """
    SinglePic capture algorithm.|
    ----------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :exposure_time: float (default: 0.1)
            | Exposure time to use
    """

    if autoscan is True:
        exposure_times = cam.color_autoscan_time() # [REQUIRED if wanting best exposure times]
        err_code = ax.LMK.iSinglePic2(max(exposure_times.items(),
                                          key=operator.itemgetter(1))[1])
    else:
        err_code = ax.LMK.iSinglePic2(exposure_time)
    ax.error_code(err_code) # Check for error

def multi_pic(autoscan=True, exposure_time=0.1, pic_count=1):
    """
    MultiPic capture algorithm.|
    ---------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :exposure_time: float (default: 0.1)
            | Exposure time to use
        :pic_count: int (default: 1)
            | Number of camera images
    """

    if autoscan is True:
        exposure_times = cam.color_autoscan_time() # [REQUIRED if wanting best exposure times]
        err_code = ax.LMK.iMultiPic2(max(exposure_times.items(), \
                                      key=operator.itemgetter(1))[1], pic_count)
    else:
        err_code = ax.LMK.iMultiPic2(exposure_time, pic_count)
    ax.error_code(err_code) # Check for error

def high_dyn_pic(autoscan=True, exposure_time=0.1, start_ratio=10.0, time_ratio=3.0, pic_count=1):
    """
    HighDyn capturing for luminance image.|
    --------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :exposure_time: float (default 0.1)
            | Exposure time to use
        :start_ratio: float (default: 10.0)
            | Exposure time is  multiplied with this parameter to determine
                the longest exposure time.
        :time_ratio: float (default: 3.0)
            | The exposure time is decreased by this factor until there is
                no overdrive in the images captured
        :pic_count: int (default: 1)
            | Number of captures for every exposure time
    """

    if autoscan is True:
        exposure_times = cam.color_autoscan_time() # [REQUIRED if wanting best exposure times]
        err_code = ax.LMK.iHighDynPic3(max(exposure_times.items(),
                                           key=operator.itemgetter(1))[1],
                                       start_ratio, time_ratio, pic_count)
    else:
        err_code = ax.LMK.iHighDynPic3(exposure_time, start_ratio, time_ratio, pic_count)
    ax.error_code(err_code) # Check for error

def color_high_dyn(autoscan=True, max_time=15.0, min_time=0.0, time_ratio=3.0, pic_count=1):
    """
    HighDyn capturing for color image.|
    ----------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :max_time: float (defualt: 5.0)
            | Largest exposure time
        :min_time: float (default: 0.0)
            | Smallest exposure time. Proposal: 0.0
        :time_ratio: float (default: 3.0)
            | Factor between two times. Proposal: 3.0
        :pic_count: int (default: 1)
            | Number of shots per integration time
    """

    if autoscan is True:
        exposure_times = cam.color_autoscan_time() # [REQUIRED if wanting best exposure times]
        err_code = ax.LMK.iColorHighDynPic2(max(exposure_times.items(),
                                                key=operator.itemgetter(1))[1],
                                            min_time, time_ratio, pic_count)
    else:
        err_code = ax.LMK.iColorHighDynPic2(max_time, min_time, time_ratio, pic_count)
    ax.error_code(err_code) # Check for error

def get_last_info():
    """
    Determine information about the preceeding capture.|
    ---------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :capture_success: int
            | 1 = successful
            | 0 = unsuccessful
        :capture_type: int
            | 1 = SinglePic
            | 2 = MutliPic
            | 3 = HighDynPic
            | 4 = ColorHighDyn
        :grey_filter: QString
            | 1 = Unused
        :color_filters: QString
            | List of color filters
        :pic_count: int
            | Number of camera images (MultiPic)
        :max_exposure_time: float
            | Maximum exposure time
        :min_exposure_time: float
            | Minimum exposure time
        :percentage_overdriven_pixels: float
            | Percentage of overdriven pixels
        :percentage_overdrive: float
            | Percentage of overdrive
        :capture_date_time: float
            | Capture date and time as float number
        :smeared_images: int
            | Number of smear images
        :modulation_frequency: float
            | Modulation frequency
    """
    last_info = {'CaptureSuccess': [], 'CaptureType': [], 'GreyFilter': [],
                 'ColorFilters': [], 'PicCount': [], 'MaxExposureTime': [],
                 'MinExposureTime': [], 'PercentageOverdrivenPixels': [],
                 'PercentageOverdrive': [], 'CaptureDateTime': [],
                 'SmearedImages': [], 'ModulationFrequency': []}

    # Messy, sorts properly into a dictionary in correct types for now.
    err_code = ax.LMK.iCaptureGetlast_info()[0]
    ax.error_code(err_code) # Check for error

    last_info['CaptureSuccess'].append(ax.LMK.iCaptureGetlast_info()[1])
    last_info['CaptureSuccess'] = str(last_info['CaptureSuccess'])[1:-1]
    last_info['CaptureSuccess'] = int(last_info['CaptureSuccess'])

    last_info['CaptureType'].append(ax.LMK.iCaptureGetlast_info()[2])
    last_info['CaptureType'] = str(last_info['CaptureType'])[1:-1]
    last_info['CaptureType'] = int(last_info['CaptureType'])

    last_info['GreyFilter'].append(ax.LMK.iCaptureGetlast_info()[3])
    last_info['GreyFilter'] = str(last_info['GreyFilter'])[1:-1]

    last_info['ColorFilters'].append(ax.LMK.iCaptureGetlast_info()[4])
    last_info['ColorFilters'] = str(last_info['ColorFilters'])[1:-1]

    last_info['PicCount'].append(ax.LMK.iCaptureGetlast_info()[5])
    last_info['PicCount'] = str(last_info['PicCount'])[1:-1]
    last_info['PicCount'] = int(last_info['PicCount'])

    last_info['MaxExposureTime'].append(ax.LMK.iCaptureGetlast_info()[6])
    last_info['MaxExposureTime'] = str(last_info['MaxExposureTime'])[1:-1]
    last_info['MaxExposureTime'] = float(last_info['MaxExposureTime'])

    last_info['MinExposureTime'].append(ax.LMK.iCaptureGetlast_info()[7])
    last_info['MinExposureTime'] = str(last_info['MinExposureTime'])[1:-1]
    last_info['MinExposureTime'] = float(last_info['MinExposureTime'])

    last_info['PercentageOverdrivenPixels'].append(ax.LMK.iCaptureGetlast_info()[8])
    last_info['PercentageOverdrivenPixels'] = str(last_info['PercentageOverdrivenPixels'])[1:-1]
    last_info['PercentageOverdrivenPixels'] = float(last_info['PercentageOverdrivenPixels'])

    last_info['PercentageOverdrive'].append(ax.LMK.iCaptureGetlast_info()[9])
    last_info['PercentageOverdrive'] = str(last_info['PercentageOverdrive'])[1:-1]
    last_info['PercentageOverdrive'] = float(last_info['PercentageOverdrive'])

    last_info['CaptureDateTime'].append(ax.LMK.iCaptureGetlast_info()[10])
    last_info['CaptureDateTime'] = str(last_info['CaptureDateTime'])[1:-1]
    last_info['CaptureDateTime'] = float(last_info['CaptureDateTime'])
    seconds = (last_info['CaptureDateTime'] - 25569) * 86400.0
    last_info['CaptureDateTime'] = datetime.datetime.utcfromtimestamp(seconds)

    last_info['SmearedImages'].append(ax.LMK.iCaptureGetlast_info()[11])
    last_info['SmearedImages'] = str(last_info['SmearedImages'])[1:-1]
    last_info['SmearedImages'] = int(last_info['SmearedImages'])

    last_info['ModulationFrequency'].append(ax.LMK.iCaptureGetlast_info()[12])
    last_info['ModulationFrequency'] = str(last_info['ModulationFrequency'])[1:-1]
    last_info['ModulationFrequency'] = float(last_info['ModulationFrequency'])

    return last_info



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 19:08:19 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

get_cameras():
set_camera():
get_lenses():
set_lens():
lenses():                   Gets list of all lenses of selected camera.
lens():                     Gets ID of selected lens.
get_focus_factors():        Gets list of all focus factors of current lens.
get_focus_factors_old():    List of available focus factors
get_focus_factor_id():      Gets ID of selected focus factor.
set_focus_factor():         Set a focus factor.
open_camera():              Set new camera calibration data.
get_converting_units():     Get converting information.
set_converting_units():     Set new converting values.
get_modulation_frequency(): Get the modulation frequency.
set_modulation_frequency(): Set the frequency of modulated light.
get_scattered_light():      Is scattered light correction switched on?
set_scattered_light():      Use of scattered light correction.
get_integration_time():     Determine current exposure time and other time parameters.
set_integration_time():     Set new exposure time.
get_max_camera_time():      Determine the maximum possible exposure time.
set_max_camera_time():      Set the maximum possible exposure time.
get_autoscan():             Get use of autoscan algorithm.
set_autoscan():             Set use of autoscan algorithm.
get_filter_wheel():         Determine filter state.
get_grey_filter_list():     List of available grey filters.
set_grey_filter_list():     Set grey filters.
set_grey_filter():          Selection of a grey filter.
color_autoscan_time():      Determine good exposure times for every color filter.
get_color_correction_list():List of available color correction factors.
set_color_correction_list():Selection of a color correction factor.
get_smear():                Get the parameter for smear correction.
set_smear():                Set the parameter for smear correction.
get_automatic():            Get the state of Automatic-Flag for exposure times.
set_automatic():            Set Automatic-Flag for all exposure times.

"""
import sys
import os
import glob
import configparser
import numpy as np
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import change_this.roots as root
import technoteam.activex as ax
import variables.dicts as dic

# Name of the TechnoTeam LMK Camera
CAMERA_NAME = str(os.listdir(root.DATA))[2:-2]

def get_cameras():
    """
    [ADD THIS]

    Returns
    -------
    cameras : TYPE
        DESCRIPTION.

    """

    print('Cameras:')
    for cameras in dic.CAMERA_NAMES.keys():
        print(cameras)

    return cameras

def set_camera(camera=None):
    """
    [ADD THIS]

    Parameters
    ----------
    camera : TYPE, optional
        DESCRIPTION. The default is None.

    Returns
    -------
    camera : TYPE
        DESCRIPTION.
    camera_no : TYPE
        DESCRIPTION.

    """

    if camera is None:
        print('Cameras:')
        for cameras in dic.CAMERA_NAMES.keys():
            print(cameras)

        camera = input('Input chosen camera: ')

    camera_no = dic.CAMERA_NAMES[camera]

    return camera, camera_no

def get_lenses(camera):
    """
    [ADD THIS]

    Parameters
    ----------
    camera : TYPE
        DESCRIPTION.

    Returns
    -------
    lenses : TYPE
        DESCRIPTION.

    """

    print('Lenses:')

    if camera == 'Old':
        for lenses in dic.OLD_LENSES.keys():
            print(lenses)
    elif camera == 'VR':
        for lenses in dic.VR_LENSES.keys():
            print(lenses)
    elif camera == 'Hyperspectral':
        for lenses in dic.HYPERSPECTRAL_LENSES.keys():
            print(lenses)

    return lenses

def set_lens(camera, lens=None):
    """
    [ADD THIS]

    Parameters
    ----------
    camera : TYPE
        DESCRIPTION.
    lens : TYPE, optional
        DESCRIPTION. The default is None.

    Returns
    -------
    lens : TYPE
        DESCRIPTION.
    lens_no : TYPE
        DESCRIPTION.

    """

    if lens is None:
        print('Lenses:')
        if camera == 'Old':
            for lenses in dic.OLD_LENSES.keys():
                print(lenses)
        elif camera == 'VR':
            for lenses in dic.VR_LENSES.keys():
                print(lenses)
        elif camera == 'Hyperspectral':
            for lenses in dic.HYPERSPECTRAL_LENSES.keys():
                print(lenses)

        lens = input('Input chosen lens: ')

    if camera == 'Old':
        lens_no = dic.OLD_LENSES[lens]
    elif camera == 'VR':
        lens_no = dic.VR_LENSES[lens]
    elif camera == 'Hyperspectral':
        lens_no = dic.HYPERSPECTRAL_LENSES[lens]


    return lens, lens_no

def lenses():
    """
    Gets list of all lenses of selected camera.|
    -------------------------------------------
    """

    global LENSES

    LENSES = []

    for lens in glob(root.DATA + '/' + CAMERA_NAME + '/*/'):
        LENSES.append(lens.split('\\')[1])

    return LENSES

def lens(current_lens='12'):
    """
    Gets ID of selected lens.|
    -------------------------
    """

    global LENS_ID

    lens = []

    for lenses in range(len(LENSES)):
        lens.append(LENSES[lenses].split('f')[1])

    if current_lens in lens:
        LENS_ID = (lens.index(current_lens))

    return LENS_ID

def get_focus_factors():
    """
    Gets list of all focus factors of current lens.|
    -----------------------------------------------
    """

    all_lenses = lenses()

    current_lens_id = lens(all_lenses)
    current_lens = all_lenses[current_lens_id]

    focus_factors = []

    config = configparser.ConfigParser()
    config.read(root.DATA + '/' + CAMERA_NAME + '/' +
                current_lens + '/' + 'FocusFactor' + dic.FILE_TYPES['ini'])
    focus_factors_size = config.get('GreyFactor', 'Size')
    focus_factors_size = int(focus_factors)
    for focus_factor in range(focus_factors_size):
        focus_factor = str(focus_factor + 1)
        focus_factors.append(config.get('GreyFactor/' + focus_factor, 'Name'))

    return focus_factors, focus_factors_size

def get_focus_factors_old():
    """
    List of available focus factors.|
    --------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :focus_factors: QStringList
            | List of focus factors
        :focus_factor: int
            | Index of current focus factor
    """
    err_code, focus_factors, focus_factor = ax.LMK.iGetFocusFactorList()
    ax.error_code(err_code) # Check for error

    return focus_factors, focus_factor

def get_focus_factor_id(focus_factors, scale='infinite'):
    """
    Gets ID of selected focus factor.|
    ---------------------------------
    """

    focus_factor = []

    for focus in range(len(focus_factors)):
        focus_factor.append(focus_factors[focus].split(' ')[2])

    if scale in focus_factor:
        focus_factor_id = (focus_factor.index(scale))

    return focus_factor_id

def set_focus_factor():
    """
    Set a focus factor.|
    -------------------
    See dialog "Camera|Recalibration" in LabSoft main menu.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :focus_factor: int
            | Index of focus factor
    """
    focus_factors, _ = get_focus_factors()
    focus_factor_id = get_focus_factor_id(focus_factors)
    err_code = ax.LMK.iSetFocusFactor(focus_factor_id)
    ax.error_code(err_code) # Check for error

def get_filter_wheels():
    """
    [ADD THIS]

    Returns
    -------
    filter_wheel_max : TYPE
        DESCRIPTION.
    filter_wheel_names : TYPE
        DESCRIPTION.

    """

    config = configparser.ConfigParser()
    config.read(root.DATA + '/' + dic.CAMERA_NAMES['Hyperspectral'] + '/' + \
                'camera' + dic.FILE_TYPES['ini'])
    filter_wheel_max = config.get('PropertyList', 'FILTER_WHEEL_MAX')
    filter_wheel_names = config.get('PropertyList', 'FILTER_WHEEL_NAMES')
    filter_wheel_names = filter_wheel_names.split(' ')
    filter_wheel_names = np.vstack(filter_wheel_names).astype(int)

    return filter_wheel_max, filter_wheel_names

def open_camera(camera_no, lens_no):
    """
    Set new camera calibration data.|
    --------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :root.DATA: QString (default: root.DATA)
            | Path to the camera calibration data. More exactly spoken path
                to the lens sub directory. After calling this function the
                camera is completely reinitialized.
            | If the string is empty a currently existing camera connection
                is finished.
    """
    err_code = ax.LMK.iSetNewCamera(root.DATA + '/' + camera_no + '/' + lens_no)
    ax.error_code(err_code) # Check for error

def get_converting_units():
    """
    Get converting information.|
    ---------------------------
    See dialog "Camera|Recalibration" in LabSoft main menu.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :units_name: QString (default: 'L')
            | Units Name used
        :units: QString (default: 'cd/m^2')
            | Units used
        :units_factor: float (default: 1.0)
            | Units factor used
    """
    err_code, units_name, units, units_factor = ax.LMK.iGetConvertingUnits()
    ax.error_code(err_code) # Check for error

    return units_name, units, units_factor

def set_converting_units(units_name='L', units='cd/m^2', units_factor=1.0):
    """
    Set new converting values.|
    --------------------------
    See dialog "Camera|Recalibration" in LabSoft main menu.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :units_name: QString (default: 'L')
            | Wished units name
        :units: QString (default: 'cd/m^2')
            | Wished units
        :units_factor: float (default: 1.0)
            | Wished units factor
    """
    err_code = ax.LMK.iSetConvertingUnits(units_name, units, units_factor)
    ax.error_code(err_code) # Check for error

def get_modulation_frequency():
    """
    Get the modulation frequency.|
    -----------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :modulation_frequency: float
            | Frequency of light source
            | 0 = no modulation is to be concerned
    """
    err_code, modulation_frequency = ax.LMK.iGetModulationFrequency()
    ax.error_code(err_code) # Check for error

    return modulation_frequency

def set_modulation_frequency(modulation_frequency=90.0):
    """
    Set the frequency of modulated light.|
    -------------------------------------
    If the light source is driven by alternating current, there are some
    restriction for the exposure times. Please inform the program about
    the modulation frequency.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :modulation_frequency: float (default: 90.0)
            | Frequency of light source
            | 0 = no modulation is to be concerned
    """
    err_code = ax.LMK.iSetModulationFrequency(modulation_frequency)
    ax.error_code(err_code) # Check for error

def get_scattered_light():
    """
    Is scattered light correction switched on?|
    ------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :ScatteredLight: int
            | 1 = scattered light correction is switched on
            | 0 = scattered light correction is switched off
    """
    err_code, scattered_light = ax.LMK.iGetScatteredLight()
    ax.error_code(err_code) # Check for error

    return scattered_light

def set_scattered_light(scattered_light=1):
    """
    Use of scattered light correction.|
    ----------------------------------
    Only usable if a parameter set is available in program.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :scattered_light: int (defualt: 1)
            | 1 = switch on scattered light correction
            | 0 = switch off scattered light correction
    """
    err_code = ax.LMK.iSetScatteredLight(scattered_light)
    ax.error_code(err_code) # Check for error

def get_integration_time():
    """
    Determine current exposure time and other parameters.|
    -----------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :current_time: float
            | Current integration time
        :previous_time: float
            | Next smaller (proposed) time
        :next_time: float
            | Next larger (proposed) time
        :min_time: float
            | Minimal possible time
        :max_time: float
            | Maximal possible time
    """
    err_code, current_time, previous_time, next_time, min_time, max_time = \
        ax.LMK.iGetIntegrationTime()
    ax.error_code(err_code) # Check for error

    return current_time, previous_time, next_time, min_time, max_time

def set_integration_time(wished_time=5.0):
    """
    Set new exposure time.|
    ----------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :wished_time: float (default: 5.0)
            | Wished integration time
    Returns
        :IntegrationTime: float
            | Realized integration time
    """
    err_code, integration_time = ax.LMK.iSetIntegrationTime(wished_time)
    ax.error_code(err_code) # Check for error

    return integration_time

def get_max_camera_time():
    """
    Determine the maximum possible exposure time.|
    ---------------------------------------------
    Normally this time is restricted by camera properties. In some cases it
    could be useful to use an even smaller maximum exposure time.
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns
        :max_time: float
            | Current maximum time
    """
    err_code, max_time = ax.LMK.iGetMaxCameraTime()
    ax.error_code(err_code) # Check for error

    return max_time

def set_max_camera_time(max_time=5.0):
    """
    Set the maximum possible exposure time.|
    ---------------------------------------
    The maximum values is of course restricted by camera properties. But
    you can use an even smaller time to avoid to long meausrement times.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :max_time: float (default: 5.0)
            | Wished value
    """
    err_code = ax.LMK.iSetMaxCameraTime(max_time)
    ax.error_code(err_code) # Check for error

def get_autoscan():
    """
    Get use of autoscan algorithm.|
    ------------------------------
    Is autoscan switched on?
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :autoscan: int
            | 1 = autoscan is switched on
            | 0 = autoscan is switched off
    """
    err_code, autoscan = ax.LMK.iGetAutoscan()
    ax.error_code(err_code) # Check for error

    return autoscan

def set_autoscan(autoscan=1):
    """
    Set use of autoscan algorithm.|
    ------------------------------
    Determination of a good exposure time before the capturing algorithm.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :autoscan: int (default: 1)
            | 1 = Use autoscan
              0 = Do not use autoscan
    """
    err_code = ax.LMK.iSetAutoscan(autoscan)
    ax.error_code(err_code) # Check for error

def get_filter_wheel():
    """
    Determine filter state.|
    -----------------------
    If there is no filter wheel, the function returns an error code.
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :current_filter_pos: int
            | Position of filter wheel
        :current_filter_name: QString
            | Name of current filter
    """
    err_code, current_filter_pos, current_filter_name = ax.LMK.iGetFilterWheel()
    ax.error_code(err_code) # Check for error

    return current_filter_pos, current_filter_name

def set_filter_wheel(filter_pos):
    """
    [ADD THIS]

    Parameters
    ----------
    filter_pos : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """

    err_code = ax.LMK.iSetFilterWheel(filter_pos)
    ax.error_code(err_code) # Check for error

def get_grey_filter_list():
    """
    List of available grey filters.|
    ------------------------------
    See dialog "Camera|Recalibration" in LabSoft main menu.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :grey_filter_list: QStringList
            | List of filters
        :grey_filter_selected: QStringList
            | 0 = none selected
            | 1 = selected
    """
    err_code, grey_filter_list, grey_filter_selected = ax.LMK.iGetGreyFilterList()
    ax.error_code(err_code) # Check for error

    return grey_filter_list, grey_filter_selected

def set_grey_filter_list(grey_filter_selected=('0', '0', '0')):
    """
    Set grey filters.|
    -----------------
    See GetGreyFilterList(). In this function, the program returns a list
    of the available grey filters and whether they are switched on or off.
    Use this function to get the appropriate list size.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :grey_filter_selected: QStringList (default: ('0', '0', '0'))
            | 0 = none selected
            | 1 = selected
    Returns:
        :grey_filter_selected: QStringList
            | 0 = none selected
            | 1 = selected
    """
    err_code, grey_filter_selected = ax.LMK.iSetGreyFilterList(grey_filter_selected)
    ax.error_code(err_code) # Check for error

    return grey_filter_selected

def set_grey_filter(grey_filter_index=0, grey_filter_selected=0):
    """
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :grey_filter_index: int (default: 0)
            | Index of filter, indexes start from '0'
        :grey_filter_selected: int (default: 0)
            | 0 = deselect
            | 1 = select
    """
    err_code = ax.LMK.iSetGreyFilter(grey_filter_index, grey_filter_selected)
    ax.error_code(err_code) # Check for error

def color_autoscan_time():
    """
    Determine good exposure times for every color filter.|
    -----------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :exposure_times: QStringList
            | Return value: List of exposure times. One time for every
                color filter.
                Attention: The list of exposure times contains one value
                for every filter in the filter wheel, independent of this
                filter is used in color capturing or not. In most cameras,
                there are at least 5 filters: Glass, X1, X2, Z, Y
                In some cases there is additionally a sixth filter available.
                The list of capture times therefore contains either 5 or 6
                entries. Color captures are mostly done with X1, X2, Z and Y
                filters. Therefore the first entry in the exposure time list
                is irrelevant for color captures. And in the case of 6
                filters,the last entry too.
                You can use the returned list of exposure times in the
                function ColorHighDyn()
    """

    color_filters = {'All': []}
    exposure_times = {'G': [], 'X1': [], 'X2': [], 'Z': [], 'VL': [], 'IR': []}

    err_code = ax.LMK.iColorAutoScanTime()[0]

    if err_code != 0:
        print('Error code:', err_code)
    else:
        color_filters['All'].append(ax.LMK.iColorAutoScanTime()[1])
        color_filters['All'] = str(color_filters['All'])[2:-2]
        temp = color_filters['All'].split(" ", 6)

        exposure_times['G'].append(temp[0])
        exposure_times['G'] = str(exposure_times['G'])[3:-4]
        exposure_times['G'] = float(exposure_times['G'])

        exposure_times['X1'].append(temp[1])
        exposure_times['X1'] = str(exposure_times['X1'])[3:-4]
        exposure_times['X1'] = float(exposure_times['X1'])

        exposure_times['X2'].append(temp[2])
        exposure_times['X2'] = str(exposure_times['X2'])[3:-4]
        exposure_times['X2'] = float(exposure_times['X2'])

        exposure_times['Z'].append(temp[3])
        exposure_times['Z'] = str(exposure_times['Z'])[3:-4]
        exposure_times['Z'] = float(exposure_times['Z'])

        exposure_times['VL'].append(temp[4])
        exposure_times['VL'] = str(exposure_times['VL'])[3:-4]
        exposure_times['VL'] = float(exposure_times['VL'])

        exposure_times['IR'].append(temp[5])
        exposure_times['IR'] = str(exposure_times['IR'])[3:-3]
        exposure_times['IR'] = float(exposure_times['IR'])


    return exposure_times

def get_color_correction_list():
    """
    List of available color correction factors.|
    -------------------------------------------
    See dialog 'Camera|Recalibration' in LabSoft main menu.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :col_corr_list: QStringList
            | List of factors
        :col_corr_selected: QStringList
            | 0 = not selected
            | 1 = selected
    """
    [err_code, col_corr_list, col_corr_selected] = ax.LMK.iGetColorCorrList()
    ax.error_code(err_code) # Check for error

    return col_corr_list, col_corr_selected

def set_color_correction(col_corr_index=0, col_corr_selected=1):
    """
    Selection of a color correction factor.|
    -------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :col_corr_index: int (default: 0)
            | List of factors
        :col_corr_selected: int (default: 1)
            | 0 = deselect
            | 1 = select
    """
    err_code = ax.LMK.iSetColorCorr(col_corr_index, col_corr_selected)
    ax.error_code(err_code) # Check for error

def get_smear():
    """
    Get the parameter for smear correction.|
    ---------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :Smear: int
            | 0 = no smear
            | !0 = smear correction with at least 10 dark images captures
            | > 10 number of dark images
    """
    err_code, smear = ax.LMK.iGetSmear()
    ax.error_code(err_code) # Check for error

    return smear

def set_smear(smear=0):
    """
    Set the parameter for smear correction.|
    ---------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :smear: int (default: 0)
            | 0 = no smear
            | !0 = smear correction with at least 10 dark images captures
            | > 10 number of dark images
    """
    err_code = ax.LMK.iSetSmear(smear)
    ax.error_code(err_code) # Check for error

def get_automatic():
    """
    Get the state of Automatic-Flag for exposure times.|
    ---------------------------------------------------
    If this flag is set, all exposure times will automatically adjusted if
    camera exposure time is reduced or enlarged.
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :automatic: int
            | 1 = option is switched on
            | 0 = option is switched off
    """
    err_code, automatic = ax.LMK.iGetAutomatic()
    ax.error_code(err_code) # Check for error

    return automatic

def set_automatic(automatic=1):
    """
    Set Automatic-Flag for all exposure times.|
    ------------------------------------------
    If this flag is set, all exposure times will automatically adjusted if
    camera exposure time is reduced or enlarged.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :automatic: int (default: 1)
            | 1 = use autoscan
            | 0 = do not use autoscan
    """
    err_code = ax.LMK.iSetAutomatic(automatic)
    ax.error_code(err_code) # Check for error

# class Select():

#     def Camera():

#         cameras = []

#         for camera in os.listdir(root.DATA):

#             cameras.append(camera)

#         return cameras

#     def Lenses():
#         """
#         Gets list of all lenses of selected camera.|
#         -------------------------------------------
#         """

#         global lenses

#         lenses = []

#         for lens in glob(root.DATA + '/' + CAMERA_NAME + '/*/'):
#             lenses.append(lens.split('\\')[1])

#         return lenses

#     def Lens(lenses, current_lens='12'):
#         """
#         Gets ID of selected lens.|
#         -------------------------
#         """

#         global lens_ID

#         lens = []

#         for l in range(len(lenses)):
#             lens.append(lenses[l].split('f')[1])

#         if current_lens in lens:
#             lens_ID = (lens.index(current_lens))

#         return lens_ID

#     def FocusFactors():
#         """
#         Gets list of all focus factors of current lens.|
#         -----------------------------------------------
#         """

#         lenses = Camera.Lenses()

#         lens_ID = Camera.Lens(lenses)
#         lens = lenses[lens_ID]

#         FocusFactors = []

#         config = configparser.ConfigParser()
#         config.read(root.DATA + '/' + CAMERA_NAME + '/' +
#                     lens + '/' + 'FocusFactor' + FILE_TYPES['ini'])
#         FocusFactors_Size = config.get('GreyFactor', 'Size')
#         FocusFactors_Size = int(FocusFactors_Size)
#         for FocusFactor in range(FocusFactors_Size):
#             FocusFactor = str(FocusFactor + 1)
#             FocusFactors.append(config.get('GreyFactor/' + FocusFactor, 'Name'))

#         return FocusFactors, FocusFactors_Size

#     def FocusFactor(FocusFactors):
#         """
#         Gets ID of selected focus factor.|
#         ---------------------------------
#         """

#         FocusFactor = []

#         for f in range(len(FocusFactors)):
#             FocusFactor.append(FocusFactors[f].split(' ')[2])

#         if SCALE in FocusFactor:
#             FocusFactor_ID = (FocusFactor.index(SCALE))

#         return FocusFactor_ID



# -*- coding: utf-8 -*-
"""
Created on Sun Feb  9 21:44:49 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

open_labsoft():             Opens the LMK4 application.
close_labsoft():            Closes the LMK4 application.
save():                     Save the measurement as a .ttcs file.
load():                     Load a measurement from a .ttcs file.
get_program_info():         Get some information about program version and
                                camera current used.

"""
import sys
import os
import datetime
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import change_this.roots as root
import variables.dicts as dic
import technoteam.activex as ax

def open_labsoft():
    """
    Opens the LMK4 application.|
    ---------------------------
    After opening, the main window is visible.
    -----------------------------------------------------------------------

    %timeit:
        2.41 s ± 75.5 ms per loop (mean ± std. dev. of 7 runs, 1 loop each)

    Returns
    -------
    None.

    """
    err_code = ax.LMK.iOpen()
    ax.error_code(err_code) # Check for error

def close_labsoft(question=0):
    """
    Closes the LMK4 application.|
    ----------------------------

    %timeit:
        23.7 ms ± 3.02 ms per loop (mean ± std. dev. of 7 runs, 1 loop each)

    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :question: int, optional
            | !0 = Opens a dialogue window in the application. The user can
                    choose whether they wish to save the current state or not
                    or cancel the closing of the program.
            | 0 = No dialogue window
    """
    err_code = ax.LMK.iClose(question)
    ax.error_code(err_code) # Check for error

def save(file_name=root.SAVE + \
         datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S') + \
         dic.FILE_TYPES['ttcs']):
    """
    Save the measurement as a .ttcs file.|
    -------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :file_name: QString, optional (default: root.SAVE + datetime + .ttcs')
            | Change to adjust root to save measurement
            | Datetime is the exact datetime of the measurement, not datetime when saved.
    """
    if not os.path.exists(root.SAVE):
        os.makedirs(root.SAVE)
    err_code = ax.LMK.iSaveProtokoll(file_name)
    ax.error_code(err_code) # Check for error

def load(file_name=root.LOAD +'Meas.ttcs'):
    """
    Load a measurement from a .ttcs file.|
    -------------------------------------

    %timeit:
        4.19 s ± 441 ms per loop (mean ± std. dev. of 7 runs, 1 loop each)

    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :file_name: QString, optional (default: 'Meas.ttcs')
            | Change to adjust root to load measurement
    """
    err_code = ax.LMK.iLoadProtokoll(file_name)
    ax.error_code(err_code) # Check for error

def get_program_info():
    """

    Get some information about program version and camera current used.|
    -------------------------------------------------------------------

    %timeit:
        144 µs ± 5.82 µs per loop (mean ± std. dev. of 7 runs, 10000 loops each)

    Parameters
    ----------
    LMK :
        Dispatch('lmk4.LMKAxServer')

    Returns
    -------
    program_type : QString
        Type of program version
    program_version : QString
        Number of program version
    camera_type : QString
        Type of camera
    camera_no : QString
        Number of camera
    lens_no : QString
        Number of lens

    """

    err_code, program_type, program_version, camera_type, camera_no, lens_no = \
        ax.LMK.iGetProgramInfo()
    ax.error_code(err_code) # Check for error

    return program_type, program_version, camera_type, camera_no, lens_no



# -*- coding: utf-8 -*-
"""
Created on Thu Mar  5 19:26:09 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

What a mess, needs to be adjusted.

"""
import sys
import os
import numpy as np
import matplotlib.pyplot as plt
import luxpy as lx
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import technoteam.evaluation as eva


def y_diff(max_ys, xyzs):
    """
    [ADD THIS]

    Parameters
    ----------
    max_ys : TYPE
        DESCRIPTION.
    xyzs : TYPE
        DESCRIPTION.

    Returns
    -------
    diff : TYPE
        DESCRIPTION.
    mean : TYPE
        DESCRIPTION.

    """

    diff = []

    for index, _ in enumerate(xyzs):
        diff.append(100 - (100 * float(xyzs[index, 1]) / float(max_ys[index
                                                                      , 0])))
    diff = np.vstack(diff)
    mean = diff.mean()

    return diff, mean

def xyz_to_u_v_(xyzs):
    """
    [ADD THIS]

    Parameters
    ----------
    xyzs : TYPE
        DESCRIPTION.

    Returns
    -------
    u_v_ : TYPE
        DESCRIPTION.

    """

    u_v_ = []
    for xyz in xyzs:
        u_v_.append(eva.xyz_to_u_v_(xyz))
    u_v_ = np.vstack(u_v_)

    return u_v_

def xyz_to_space(input_type='RGB', R=None, G=None, B=None, W=None,
                 file='GW_2019-09-13.txt', space='uv'):
    """
    [ADD THIS]

    Parameters
    ----------
    input_type : TYPE, optional
        DESCRIPTION. The default is 'RGB'.
    R : TYPE, optional
        DESCRIPTION. The default is None.
    G : TYPE, optional
        DESCRIPTION. The default is None.
    B : TYPE, optional
        DESCRIPTION. The default is None.
    W : TYPE, optional
        DESCRIPTION. The default is None.
    file : TYPE, optional
        DESCRIPTION. The default is 'GW_2019-09-13.txt'.
    space : TYPE, optional
        DESCRIPTION. The default is 'uv'.

    Returns
    -------
    TYPE
        DESCRIPTION.

    """

    if input_type == 'RGB':
        R_Yuv = lx.xyz_to_Yuv(R)
        G_Yuv = lx.xyz_to_Yuv(G)
        B_Yuv = lx.xyz_to_Yuv(B)
        return R_Yuv, G_Yuv, B_Yuv

    elif input_type == 'RGBW':
        R_Yuv = lx.xyz_to_Yuv(R)
        G_Yuv = lx.xyz_to_Yuv(G)
        B_Yuv = lx.xyz_to_Yuv(B)
        W_Yuv = lx.xyz_to_Yuv(W)
        return R_Yuv, G_Yuv, B_Yuv, W_Yuv

    elif input_type == 'file':
        XYZs = np.loadtxt(file, delimiter='\t')
        if space == 'uv':
            output = lx.xyz_to_Yuv(XYZs)
        else:
            output = lx.xyz_to_Yxy(XYZs)
        for count, _ in enumerate(output):
            count = count + 1
        return XYZs, output, count

def input_to_gamut(input_type='RGB', count=None, output=None, R=None,
                   G=None, B=None, C=None, M=None, Y=None):
    """
    [ADD THIS]

    Parameters
    ----------
    input_type : TYPE, optional
        DESCRIPTION. The default is 'RGB'.
    count : TYPE, optional
        DESCRIPTION. The default is None.
    output : TYPE, optional
        DESCRIPTION. The default is None.
    R : TYPE, optional
        DESCRIPTION. The default is None.
    G : TYPE, optional
        DESCRIPTION. The default is None.
    B : TYPE, optional
        DESCRIPTION. The default is None.
    C : TYPE, optional
        DESCRIPTION. The default is None.
    M : TYPE, optional
        DESCRIPTION. The default is None.
    Y : TYPE, optional
        DESCRIPTION. The default is None.

    Returns
    -------
    u_out : TYPE
        DESCRIPTION.
    v_out : TYPE
        DESCRIPTION.

    """

    if input_type == 'RGB':
        u_out, v_out = np.array([[R[:, 1], G[:, 1],
                                  B[:, 1], R[:, 1]],
                                 [R[:, 2], G[:, 2],
                                  B[:, 2], R[:, 2]]])
        return u_out, v_out

    elif input_type == 'RGBCMY':
        u_out, v_out = np.array([[R[:, 1], Y[:, 1],
                                  G[:, 1], C[:, 1],
                                  B[:, 1], M[:, 1],
                                  R[:, 1]],
                                 [R[:, 2], Y[:, 2],
                                  G[:, 2], C[:, 2],
                                  B[:, 2], M[:, 2],
                                  R[:, 2]]])
        return u_out, v_out

    elif input_type == 'file':
        if count == 4:
            u_out, v_out = np.array([[output[0, 1], output[1, 1],
                                      output[2, 1], output[0, 1]],
                                     [output[0, 2], output[1, 2],
                                      output[2, 2], output[0, 2]]])
            return u_out, v_out

        elif count == 7 or count == 8:
            u_out, v_out = np.array([[output[0, 0], output[6, 0], output[1, 0],
                                      output[4, 0], output[2, 0], output[5, 0],
                                      output[0, 0]],
                                     [output[0, 1], output[6, 1], output[1, 1],
                                      output[4, 1], output[2, 1], output[5, 1],
                                      output[0, 1]]])
            return u_out, v_out

        elif count == 13 or count == 14:
            u_out, v_out = np.array([[output[0, 1], output[1, 1], output[2, 1],
                                      output[3, 1], output[4, 1], output[5, 1],
                                      output[6, 1], output[7, 1], output[8, 1],
                                      output[9, 1], output[10, 1], output[11, 1],
                                      output[0, 1]],
                                     [output[0, 2], output[1, 2], output[2, 2],
                                      output[3, 2], output[4, 2], output[5, 2],
                                      output[6, 2], output[7, 2], output[8, 2],
                                      output[9, 2], output[10, 2], output[11, 2],
                                      output[0, 2]]])
            return u_out, v_out

        elif count == 84:
            u_out, v_out = np.array([[output[26, 1], output[83, 1], output[39, 1],
                                      output[75, 1], output[52, 1], output[67, 1],
                                      output[26, 1]],
                                     [output[26, 2], output[83, 2], output[39, 2],
                                      output[75, 2], output[52, 2], output[67, 2],
                                      output[26, 2]]])

            return u_out, v_out

def show(input_type='file', space='uv', count=None, output=None,
         new=True, color='k', label=None):
    """
    [ADD THIS]

    Parameters
    ----------
    input_type : TYPE, optional
        DESCRIPTION. The default is 'file'.
    space : TYPE, optional
        DESCRIPTION. The default is 'uv'.
    count : TYPE, optional
        DESCRIPTION. The default is None.
    output : TYPE, optional
        DESCRIPTION. The default is None.
    new : TYPE, optional
        DESCRIPTION. The default is True.
    color : TYPE, optional
        DESCRIPTION. The default is 'k'.
    label : TYPE, optional
        DESCRIPTION. The default is None.

    Returns
    -------
    None.

    """

    x, y = input_to_gamut(input_type=input_type, count=count, output=output)

    if space == 'uv':
        uv(x, y, gamut=True, label=label, title=None, color=color, new=new)

def uv(u_, v_, gamut=None, label='u_, v_', facecolors='none', color='k',
       linestyle='--', title='u_, v_', grid=True, new=False):
    """
    Plot u', v' color coordinates using Luxpy.|
    ------------------------------------------

    Args:
        :u_:
            u' coordinate(s) - must be float, int, or array
        :v_:
            v' coordinate(s) - must be float, int, or array
        :gamut:
            None
            Anything bar 'None' will assume a gamut is going to be created.
            This expects a plot in the order of a line in the diagram e.g.
            R, G, B, R or R, Y, G, C, B, M, R. Must go back to starting
            value to complete gamut line.
        :label:
            'u_, v_'
            Change to adjust label within diagram of the input.
        :facecolors:
            'none'
            Change to adjust face color of value within diagram. Only if
            gamut=None
        :color:
            'k'
            Change to adjust color of either edge color or line color,
            depending on if 'gamut' is chosen.
        :linestyle:
            '--'
            Change to adjust style of line if gamut is not None.
        :title:
            'u_, v_'
            Change to adjust title of figure.
        :grid:
            True
            Change to 'None' for no grid in diagram.
        :kwargs:
            Additional keyword arguments for use with matplotlib.pyplot

    Returns:

    """
    if new is True:
        plt.figure()
        ax_uv = plt.axes()
        lx.plot_chromaticity_diagram_colors(256, 0.3, 1, lx._CIEOBS, 'Yuv',
                                            {}, True, ax_uv, grid,
                                            'Times New Roman', 12)
    else:
        ax_uv = plt.axes()

    if gamut is None:
        plt.scatter(float(u_), float(v_), label=label, facecolors=facecolors,
                    edgecolors=color)
    else:
        ax_uv.plot(u_, v_, label=label, color=color, linestyle=linestyle)
    ax_uv.set_title(title)
    ax_uv.set_xlim([-0.1, 0.7])
    ax_uv.set_ylim([-0.1, 0.7])
    ax_uv.legend()



# -*- coding: utf-8 -*-
"""
Created on Sun Feb  9 21:38:59 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

error_code():               Prints error code if error occurs.

"""
from win32com.client import Dispatch

LMK = Dispatch('lmk4.LMKAxServer')

def error_code(err_code):
    """
    Prints error code if error occurs.|
    ----------------------------------

    %timeit:
        563 µs ± 15.6 µs per loop (mean ± std. dev. of 7 runs, 1000 loops each)

    Parameters:
        :err_code: int
            | Error code from ActiveX server.
            | 0 = Continue with no error.
            | !0 = Error with function from ActiveX server.
    """
    if err_code != 0:
        print('Error code:', err_code, '\n')
        _, error_info = LMK.iGetErrorInformation()
        print('Error info:', error_info)



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 19:22:05 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

get_value_unit():           Get the values and units of the axis.
set_value_unit():           Set the values and units of the axis.

"""
import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import technoteam.activex as ax
import variables.dicts as dic

def get_value_unit(image=dic.IMAGE_TYPES['Color']):
    """
    Get the values and units of the axis.|
    -------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :Image: int (default: IMAGE_TYPES['Color'])
            | Index of image
    Returns:
        :x_value: QString
            | Value of X axis
        :x_unit: QString
            | Unit of X axis
        :y_value: QString
            | Value of Y axis
        :y_unit: QString
            | Unit of Y axis
        :unit_area: QString
            | Unit of area
    """
    [err_code, x_value, x_unit, y_value, y_unit, unit_area] = ax.LMK.iCoordSystemGetValueUnit(image)
    ax.error_code(err_code) # Check for error

    return x_value, x_unit, y_value, y_unit, unit_area

def set_value_unit(image=dic.IMAGE_TYPES['Color'], x_value='x',
                   x_unit='pix', y_value='y', y_unit='pix', unit_area='pix^2'):
    """
    Set the values and units of the axis.|
    -------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: IMAGE_TYPES['Color'])
            | Index of image
        :x_value: QString (default: 'x')
            | Value of X axis
        :x_unit: QString (default: 'pix')
            | Unit of X axis
        :y_value: QString (default: 'y')
            | Value of Y axis
        :y_unit: QString (default: 'pix')
            | Unit of Y axis
        :unit_area: QString (default: 'pix^2')
            | Unit of area
    """
    err_code = ax.LMK.iCoordSystemSetValueUnit(image, x_value, x_unit, y_value, y_unit, unit_area)
    ax.error_code(err_code) # Check for error



# -*- coding: utf-8 -*-
"""
Created on Sun Feb  9 21:37:15 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

# get_number():               Returns the number of tables in the result tab widget.
# get_name_and_caption():     Returns the name and the caption of an existing table.
# get_index():                Search for a table, given by name or caption.
# get_number_columns():       Returns the number of columns of a table.
# get_number_lines():         Returns the number of lines of a table.
# get_column():               Returns the column header of a column of a table.
# get_cell():                 Returns the content of a cell of a table.
# get_all_content():          Returns all cells of a table as a list.

"""
import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import technoteam.activex as ax

def get_number():
    """
    Returns the number of tables in the result tab widget.|
    ------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
    Returns:
        :tables_no: int
            | Returns the number of tables
    """
    err_code, tables_no = ax.LMK.iTableGetNumber()
    ax.error_code(err_code) # Check for error

    return tables_no

def get_name_and_caption(table_id=2):
    """
    Returns the name and the caption of an existing table.|
    ------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :table_id: int (default: 2)
            | Index of wished table
    Returns:
        :table_name: QStringList
            | Returns the name of the table
        :table_caption: QStringList
            | Returns the caption of the table
    """
    err_code, table_name, table_caption = ax.LMK.iTableGetNameAndCaption(table_id)
    ax.error_code(err_code) # Check for error

    return table_name, table_caption

def get_index(table_name_or_caption='Last capture'):
    """
    Search for a table, given by name or caption.|
    ---------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :table_name_or_caption: QString (default: 'Last capture')
            | Searched name or caption of table
    Returns:
        :table_id: int (default: 2)
            | Index of table
            | -1 = not found
    """
    err_code, table_id = ax.LMK.iTableGetIndex(table_name_or_caption)
    ax.error_code(err_code) # Check for error

    return table_id

def get_number_columns(table_id=2):
    """
    Returns the number of columns of a table.|
    -----------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :table_id: int (default: 2)
            | Index of wished table
    Returns:
        :table_number_columns: int
            | Returns the number of columns in table
    """
    err_code, table_number_columns = ax.LMK.iTableGetNumberColumns(table_id)
    ax.error_code(err_code) # Check for error

    return table_number_columns

def get_number_lines(table_id=2):
    """
    Returns the number of lines of a table.|
    ---------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :table_id: int (default: 2)
            | Index of wished table
    Returns:
        :table_number_lines: int
            | Returns the number of lines in table
    """
    err_code, table_number_lines = ax.LMK.iTableGetNumberLines(table_id)
    ax.error_code(err_code) # Check for error

    return table_number_lines

def get_column(table_id=2, table_column_id=0):
    """
    Returns the column header of a column of a table.|
    -------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :table_id: int (default: 2)
            | Index of wished table
        :table_column_id: int (default: 0)
            | Index of wished column
    Returns:
        :table_column_name: QString
            | Returns the name of the column
        :table_column_caption: QString
            | Returns the caption of the column
        :table_column_unit: QString
            | Returns the unit of the column
    """
    err_code, table_column_name, table_column_caption,\
        table_column_unit = ax.LMK.iTableGetColumn(table_id, table_column_id)
    ax.error_code(err_code) # Check for error

    return table_column_name, table_column_caption, table_column_unit

def get_cell(table_id=2, table_line_id=1, table_column_id=8):
    """
    Returns the content of a cell of a table.|
    -----------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :table_id: int (default: 2)
            | Index of wished table
        :table_line_id: int (default: 1)
            | Index of wished table line
        :table_column_id: int (default: 8)
            | Index of wished table column
    Returns:
        :table_cell: QString
            | Returns the content of table cell
    """
    err_code, table_cell = ax.LMK.iTableGetCell(table_id, table_line_id, table_column_id)
    ax.error_code(err_code) # Check for error

    return table_cell

def get_all_content(table_id=2):
    """
    Returns all cells of a table as a list.|
    ---------------------------------------
    The entries are sorted line by line.
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :table_id: int (default: 2)
            | Index of wished table
    Returns:
        :table_content: QStringList
            | List with content of all cells
    """
    [err_code, table_content] = ax.LMK.iGetAllContent(table_id)
    ax.error_code(err_code) # Check for error

    return table_content



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 19:42:21 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

statistic_exists():         Proof, if a statistic for a image/region/statistic type
create_statistic():         Create a new statistic.
get_standard_statistic():   Determine parameter of the standard statistic.
delete_statistic():         Delete an existing statistic.
proj_rect_lum():            Projective Rectifaction of Luminance Image.
proj_rect_col():            Projective Rectifaction of Color Image.
coord_trans_lum():          Coordinate Transformation of Luminance Image.
coord_trans_col():          Coordinate Transformation of Color Image.
get_xyz():
get_max_lum():
get_image_mean_xyz():       Create region size of image and get mean XYZ.
get_circle_mean_xyz():      Get size of image, create eclipse in center,
                                get mean XYZ from circle.
# get_grid_mean_xyz():        Create regions as a grid in image and get mean XYZ.
# get_color_hist_vals():      Get the values of the histogram in a color image.
get_pixel_color():          Get a pixel value of a color image.
convert_cie_rgb():          Conversion of a color value from CIE-RGB to another color space.
xyz_to_xy():                Convert XYZ to x, y.
xyz_to_u_v_():              Convert XYZ to u', v'.
show_xy():                  Plot x, y color coordinates using Luxpy.
show_u_v_():                Plot u', v' color coordinates using Luxpy.

"""
import sys
import os
import numpy as np
from matplotlib import pyplot as plt
import luxpy as lx
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import variables.dicts as dic
import technoteam.activex as ax
import technoteam.image as im
import technoteam.region as reg
import technoteam.table as tab

def statistic_exists(image=dic.IMAGE_TYPES['Color'], region=0):
    """
    Proof, if a statistic exists for a image / region / statistic type.|
    -------------------------------------------------------------------

    %timeit:
        138 µs ± 7.29 µs per loop (mean ± std. dev. of 7 runs, 10000 loops each)

    Parameters:
        :lmk:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: Image = IMAGE_TYPES['Color'])
            | Image
        :region: int (default: 0)
            | Index of region in this image
    Returns:
        :exists: int
            | 1 = statistic exists
            | 0 = statistic does not exist
        :statistic_type: int
            | Index of statistic type
        :statistic_index: int
            | Index in statistic list?
    """
    _, exists, statistic_type, statistic_index = ax.LMK.iHaveStatistic(image, region)

    return exists, statistic_type, statistic_index

def create_statistic(statistic_type=dic.STATISTIC_TYPES['standardColor'],
                     image=dic.IMAGE_TYPES['Color'], region=0,
                     num_param=1, param_list=[1]):
    """
    Create a new statistic.|
    -----------------------
    Parameters:
        :lmk:
            | Dispatch('lmk4.LMKAxServer')
        :statistic_type: int (default: STATISTIC_TYPES['standardColor'])
            | Index of statistic type
        :image: int (default: Image = IMAGE_TYPES['Color'])
            | Image
        :region: int (default: 0)
            | Index of region in this image
        :num_param: int (default: 1)
            | Number of parameters for this statistic
        :param_list: QStringList
    """
    [err_code, statistic] = ax.LMK.iCreateStatistic(statistic_type, image, region,
                                                    num_param, param_list)
    ax.error_code(err_code) # Check for error

    return statistic

def get_standard_statistic(statistic_type=dic.STATISTIC_TYPES['standardColor'],
                           region=0, color_class=0):
    """
    Determine parameter of the standard statistic.|
    ----------------------------------------------
    Parameters:
        :lmk:
            | Dispatch('lmk4.LMKAxServer')
        :statistic_type: int (default: STATISTIC_TYPES['standardColor'])
            | Index of statistic type
        :region: int (default: 0)
            | Index of region in this image
        :color_class: int (default: 0)
            | Index of class or color:
                0 = blue
                1 = green
                2 = red
    Returns:
        :area: float
            | Amount of pixels in area of statistic
        :stat_min: float
            | Minimum value
        :stat_max: float
            | Maximum value
        :stat_mean: float
            | Mean value
        :variance: float
            | Variance (...SD?) in values
    """

    stats = {'Area': [], 'Min': [], 'Max': [], 'Mean': [], 'Variance': []}

    err_code, _, _, _, _, _ = ax.LMK.iGetStandardStatistic2(statistic_type, region, color_class)
    ax.error_code(err_code) # Check for error

    stats['Area'].append(ax.LMK.iGetStandardStatistic2(statistic_type, region, color_class)[1])
    stats['Min'].append(ax.LMK.iGetStandardStatistic2(statistic_type, region, color_class)[2])
    stats['Max'].append(ax.LMK.iGetStandardStatistic2(statistic_type, region, color_class)[3])
    stats['Mean'].append(ax.LMK.iGetStandardStatistic2(statistic_type, region, color_class)[4])
    stats['Variance'].append(ax.LMK.iGetStandardStatistic2(statistic_type, region, color_class)[5])

    return stats

def delete_statistic(statistic_type=dic.STATISTIC_TYPES['standardColor'], stat_no=0):
    """
    Delete an existing statistic.|
    -----------------------------
    Parameters:
        :lmk:
            | Dispatch('lmk4.LMKAxServer')
        :statistic_type: int (default: STATISTIC_TYPES['standardColor'])
            | Index of statistic type
        :stat_no: int (default: 0)
            | Index of statistic number
    """
    err_code = ax.LMK.iDeleteStatistic(statistic_type, stat_no)
    ax.error_code(err_code) # Check for error

def proj_rect_lum():
    """
    [ADD THIS]

    Returns
    -------
    None.

    """
    ax.LMK.iExecMenuPoint('Macros|ProjRectLum')

def proj_rect_col():
    """
    [ADD THIS]

    Returns
    -------
    None.

    """
    ax.LMK.iExecMenuPoint('Macros|ProjRectCol')

def coord_trans_lum():
    """
    [ADD THIS]

    Returns
    -------
    None.

    """
    ax.LMK.iExecMenuPoint('Macros|CoordTransLum')

def coord_trans_col():
    """
    [ADD THIS]

    Returns
    -------
    None.

    """
    ax.LMK.iExecMenuPoint('Macros|CoordTransCol')


def get_xyz(index_out):
    """
    [ADD THIS]

    Returns
    -------
    None.

    """

    blue_stats = get_standard_statistic(region=index_out, color_class=0)
    b_mean = str(blue_stats['Mean'])[1:-1]
    b_mean = float(b_mean)
    green_stats = get_standard_statistic(region=index_out, color_class=1)
    g_mean = str(green_stats['Mean'])[1:-1]
    g_mean = float(g_mean)
    red_stats = get_standard_statistic(region=index_out, color_class=2)
    r_mean = str(red_stats['Mean'])[1:-1]
    r_mean = float(r_mean)

    output_color = convert_cie_rgb(cie_r=r_mean, cie_g=g_mean, cie_b=b_mean)

    return output_color

def get_max_lum():
    """
    [ADD THIS]

    Returns
    -------
    output : TYPE
        DESCRIPTION.

    """

    max_lum = tab.get_cell(table_id=2, table_line_id=0, table_column_id=6)
    output = np.float64(max_lum)

    return output

def get_image_mean_xyz():
    """
    Create region size of image and get mean XYZ.|
    ----------------------------------------------
    """

    # Create a region the size of the whole image
    reg.create_rect_image_size()
    # Get ID of region
    _, index_out = reg.get_id(dic.IMAGE_TYPES['Color'], name='1')
    # Select region from index of region
    reg.select(index=index_out)

    ### Evaluate Region
    create_statistic(dic.STATISTIC_TYPES['standardColor'], dic.IMAGE_TYPES['Color'],
                     index_out, param_lis=[1])

    blue_stats = get_standard_statistic(color_class=0)
    b_mean = str(blue_stats['Mean'])[1:-1]
    b_mean = float(b_mean)
    green_stats = get_standard_statistic(color_class=1)
    g_mean = str(green_stats['Mean'])[1:-1]
    g_mean = float(g_mean)
    red_stats = get_standard_statistic(color_class=2)
    r_mean = str(red_stats['Mean'])[1:-1]
    r_mean = float(r_mean)

    output_color = convert_cie_rgb(cie_r=r_mean, cie_g=g_mean, cie_b=b_mean)

    return output_color

def get_circle_mean_xyz():
    """
    Get size of image, create eclipse in center, get mean XYZ from circle.|
    ----------------------------------------------------------------------
    """

    # Get size of image
    _, image_last_line, _, image_last_col, _ = im.get_size()

    # If region already exists, delete it
    err_code, index_out = reg.get_id()
    if err_code == 0:
        reg.delete()

    # If statistic already exists, delete it
    exists, _, _ = statistic_exists()
    if exists == 1:
        delete_statistic()

    # Create a region the size of the whole image
    reg.create(x_coords=[image_last_col/2, 1700, 1700],
               y_coords=[image_last_line/2, 1700, 1700])
    # Get ID of region
    err_code, index_out = reg.get_id(dic.IMAGE_TYPES['Color'], name='1')
    # Select region from index of region
    reg.select(index=index_out)

    ### Evaluate Region
    create_statistic(dic.STATISTIC_TYPES['standardColor'], dic.IMAGE_TYPES['Color'],
                     index_out, param_list=[1])

    blue_stats = get_standard_statistic(color_class=0)
    b_mean = str(blue_stats['Mean'])[1:-1]
    b_mean = float(b_mean)
    green_stats = get_standard_statistic(color_class=1)
    g_mean = str(green_stats['Mean'])[1:-1]
    g_mean = float(g_mean)
    red_stats = get_standard_statistic(color_class=2)
    r_mean = str(red_stats['Mean'])[1:-1]
    r_mean = float(r_mean)

    output_color = convert_cie_rgb(cie_r=r_mean, cie_g=g_mean, cie_b=b_mean)

    return output_color

# def get_grid_mean_xyz():
#     """
#     Create regions as a grid in image and get mean XYZ.|
#     ---------------------------------------------------
#     """

#     # Create a region the size of the whole image
#     reg.CreateGrid(LMK)
#     # Get ID of region
#     err_code, Index_Zero = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '1')
#     err_code, Index_One = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '2')
#     err_code, Index_Two = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '3')
#     err_code, Index_Three = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '4')
#     err_code, Index_Four = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '5')
#     err_code, Index_Five = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '6')
#     err_code, Index_Six = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '7')
#     err_code, Index_Seven = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '8')
#     err_code, Index_Eight = reg.GetID(LMK, dic.IMAGE_TYPES['Color'], Name = '9')
#     # Select region from index of region
#     reg.Select(LMK, Index = Index_Zero)
#     reg.Select(LMK, Index = Index_One)
#     reg.Select(LMK, Index = Index_Two)
#     reg.Select(LMK, Index = Index_Three)
#     reg.Select(LMK, Index = Index_Four)
#     reg.Select(LMK, Index = Index_Five)
#     reg.Select(LMK, Index = Index_Six)
#     reg.Select(LMK, Index = Index_Seven)
#     reg.Select(LMK, Index = Index_Eight)

    ## Evaluate Region
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_Zero, ParamList = [1])
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_One, ParamList = [1])
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_Two, ParamList = [1])
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_Three, ParamList = [1])
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_Four, ParamList = [1])
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_Five, ParamList = [1])
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_Six, ParamList = [1])
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_Seven, ParamList = [1])
    # CreateStatistic(LMK, dic.STATISTIC_TYPES['standardColor'],
    #                 dic.IMAGE_TYPES['Color'], Index_Eight, ParamList = [1])

#     Blue_Stats_Zero = GetStandardStatistic(LMK, Region = 0, Class = 0)
#     B_Mean_Zero = str(Blue_Stats_Zero['Mean'])[1:-1]
#     B_Mean_Zero = float(B_Mean_Zero)
#     Green_Stats_Zero = GetStandardStatistic(LMK, Region = 0, Class = 1)
#     G_Mean_Zero = str(Green_Stats_Zero['Mean'])[1:-1]
#     G_Mean_Zero = float(G_Mean_Zero)
#     Red_Stats_Zero = GetStandardStatistic(LMK, Region = 0, Class = 2)
#     R_Mean_Zero = str(Red_Stats_Zero['Mean'])[1:-1]
#     R_Mean_Zero = float(R_Mean_Zero)

#     Blue_Stats_One = GetStandardStatistic(LMK, Region = 1, Class = 0)
#     B_Mean_One = str(Blue_Stats_One['Mean'])[1:-1]
#     B_Mean_One = float(B_Mean_One)
#     Green_Stats_One = GetStandardStatistic(LMK, Region = 1, Class = 1)
#     G_Mean_One = str(Green_Stats_One['Mean'])[1:-1]
#     G_Mean_One = float(G_Mean_One)
#     Red_Stats_One = GetStandardStatistic(LMK, Region = 1, Class = 2)
#     R_Mean_One = str(Red_Stats_One['Mean'])[1:-1]
#     R_Mean_One = float(R_Mean_One)

#     Blue_Stats_Two = GetStandardStatistic(LMK, Region = 2, Class = 0)
#     B_Mean_Two = str(Blue_Stats_Two['Mean'])[1:-1]
#     B_Mean_Two = float(B_Mean_Two)
#     Green_Stats_Two = GetStandardStatistic(LMK, Region = 2, Class = 1)
#     G_Mean_Two = str(Green_Stats_Two['Mean'])[1:-1]
#     G_Mean_Two = float(G_Mean_Two)
#     Red_Stats_Two = GetStandardStatistic(LMK, Region = 2, Class = 2)
#     R_Mean_Two = str(Red_Stats_Two['Mean'])[1:-1]
#     R_Mean_Two = float(R_Mean_Two)

#     Blue_Stats_Three = GetStandardStatistic(LMK, Region = 3, Class = 0)
#     B_Mean_Three = str(Blue_Stats_Three['Mean'])[1:-1]
#     B_Mean_Three = float(B_Mean_Three)
#     Green_Stats_Three = GetStandardStatistic(LMK, Region = 3, Class = 1)
#     G_Mean_Three = str(Green_Stats_Three['Mean'])[1:-1]
#     G_Mean_Three = float(G_Mean_Three)
#     Red_Stats_Three = GetStandardStatistic(LMK, Region = 3, Class = 2)
#     R_Mean_Three = str(Red_Stats_Three['Mean'])[1:-1]
#     R_Mean_Three = float(R_Mean_Three)

#     Blue_Stats_Four = GetStandardStatistic(LMK, Region = 4, Class = 0)
#     B_Mean_Four = str(Blue_Stats_Four['Mean'])[1:-1]
#     B_Mean_Four = float(B_Mean_Four)
#     Green_Stats_Four = GetStandardStatistic(LMK, Region = 4, Class = 1)
#     G_Mean_Four = str(Green_Stats_Four['Mean'])[1:-1]
#     G_Mean_Four = float(G_Mean_Four)
#     Red_Stats_Four = GetStandardStatistic(LMK, Region = 4, Class = 2)
#     R_Mean_Four = str(Red_Stats_Four['Mean'])[1:-1]
#     R_Mean_Four = float(R_Mean_Four)

#     Blue_Stats_Five = GetStandardStatistic(LMK, Region = 5, Class = 0)
#     B_Mean_Five = str(Blue_Stats_Five['Mean'])[1:-1]
#     B_Mean_Five = float(B_Mean_Five)
#     Green_Stats_Five = GetStandardStatistic(LMK, Region = 5, Class = 1)
#     G_Mean_Five = str(Green_Stats_Five['Mean'])[1:-1]
#     G_Mean_Five = float(G_Mean_Five)
#     Red_Stats_Five = GetStandardStatistic(LMK, Region = 5, Class = 2)
#     R_Mean_Five = str(Red_Stats_Five['Mean'])[1:-1]
#     R_Mean_Five = float(R_Mean_Five)

#     Blue_Stats_Six = GetStandardStatistic(LMK, Region = 6, Class = 0)
#     B_Mean_Six = str(Blue_Stats_Six['Mean'])[1:-1]
#     B_Mean_Six = float(B_Mean_Six)
#     Green_Stats_Six = GetStandardStatistic(LMK, Region = 6, Class = 1)
#     G_Mean_Six = str(Green_Stats_Six['Mean'])[1:-1]
#     G_Mean_Six = float(G_Mean_Six)
#     Red_Stats_Six = GetStandardStatistic(LMK, Region = 6, Class = 2)
#     R_Mean_Six = str(Red_Stats_Six['Mean'])[1:-1]
#     R_Mean_Six = float(R_Mean_Six)

#     Blue_Stats_Seven = GetStandardStatistic(LMK, Region = 7, Class = 0)
#     B_Mean_Seven = str(Blue_Stats_Seven['Mean'])[1:-1]
#     B_Mean_Seven = float(B_Mean_Seven)
#     Green_Stats_Seven = GetStandardStatistic(LMK, Region = 7, Class = 1)
#     G_Mean_Seven = str(Green_Stats_Seven['Mean'])[1:-1]
#     G_Mean_Seven = float(G_Mean_Seven)
#     Red_Stats_Seven = GetStandardStatistic(LMK, Region = 7, Class = 2)
#     R_Mean_Seven = str(Red_Stats_Seven['Mean'])[1:-1]
#     R_Mean_Seven = float(R_Mean_Seven)

#     Blue_Stats_Eight = GetStandardStatistic(LMK, Region = 8, Class = 0)
#     B_Mean_Eight = str(Blue_Stats_Eight['Mean'])[1:-1]
#     B_Mean_Eight = float(B_Mean_Eight)
#     Green_Stats_Eight = GetStandardStatistic(LMK, Region = 8, Class = 1)
#     G_Mean_Eight = str(Green_Stats_Eight['Mean'])[1:-1]
#     G_Mean_Eight = float(G_Mean_Eight)
#     Red_Stats_Eight = GetStandardStatistic(LMK, Region = 8, Class = 2)
#     R_Mean_Eight = str(Red_Stats_Eight['Mean'])[1:-1]
#     R_Mean_Eight = float(R_Mean_Eight)

    # Output_Color_Zero = Convert_CIE_RGB(LMK, CIE_R = R_Mean_Zero,
    #                                     CIE_G = G_Mean_Zero, CIE_B = B_Mean_Zero)
    # Output_Color_One = Convert_CIE_RGB(LMK, CIE_R = R_Mean_One,
    #                                    CIE_G = G_Mean_One, CIE_B = B_Mean_One)
    # Output_Color_Two = Convert_CIE_RGB(LMK, CIE_R = R_Mean_Two,
    #                                    CIE_G = G_Mean_Two, CIE_B = B_Mean_Two)
    # Output_Color_Three = Convert_CIE_RGB(LMK, CIE_R = R_Mean_Three,
    #                                      CIE_G = G_Mean_Three, CIE_B = B_Mean_Three)
    # Output_Color_Four = Convert_CIE_RGB(LMK, CIE_R = R_Mean_Four,
    #                                     CIE_G = G_Mean_Four, CIE_B = B_Mean_Four)
    # Output_Color_Five = Convert_CIE_RGB(LMK, CIE_R = R_Mean_Five,
    #                                     CIE_G = G_Mean_Five, CIE_B = B_Mean_Five)
    # Output_Color_Six = Convert_CIE_RGB(LMK, CIE_R = R_Mean_Six,
    #                                    CIE_G = G_Mean_Six, CIE_B = B_Mean_Six)
    # Output_Color_Seven = Convert_CIE_RGB(LMK, CIE_R = R_Mean_Seven,
    #                                      CIE_G = G_Mean_Seven, CIE_B = B_Mean_Seven)
    # Output_Color_Eight = Convert_CIE_RGB(LMK, CIE_R = R_Mean_Eight,
    #                                      CIE_G = G_Mean_Eight, CIE_B = B_Mean_Eight)

    # return Output_Color_Zero, Output_Color_One, Output_Color_Two, \
    #     Output_Color_Three, Output_Color_Four, Output_Color_Five, \
    #         Output_Color_Six, Output_Color_Seven, Output_Color_Eight

def get_color_hist_vals(image=dic.IMAGE_TYPES['Color'],
                        color_space=dic.COLOR_SPACES['XYZ']):
    """
    Get the values of the histogram in a color image.|
    -------------------------------------------------
    The color space is always RGB.
    -----------------------------------------------------------------------
    Parameters:
        :lmk:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: IMAGE_TYPES['Color'])
            | Index of object
        :color_space: int (default: COLOR_SPACES['XYZ'])
            | Wished color space from COLOR_SPACES{}
    Returns:
        :num_param: int
            | Number of values
        :x_coords: QStringList
            | x-coordinates
        :hist_values: QStringList
            | Histogram values
    """
    err_code, num_param, x_coords, hist_values \
        = ax.LMK.iGetColorHistogramValues(image, color_space)
    ax.error_code(err_code) # Check for error

    return num_param, x_coords, hist_values

def get_pixel_color(image=dic.IMAGE_TYPES['Color'], line=500, column=500):
    """
    Get a pixel value of a color image.|
    -----------------------------------
    Parameters:
        :lmk:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: IMAGE_TYPES['Color'])
            | Index of image
        :line: int (default: 500)
            | Line index
        :column: int (default: 500)
            | Column index
    Returns:
        | The function returns an error if the pixel position is
            outside or the image is a color image.
        :cie_r: float
            | Gets the red component of the pixel value
        :cie_g: float
            | Gets the green component of the pixel value
        :cie_b: float
            | Gets the blue component of the pixel value
    """
    err_code, cie_r, cie_g, cie_b = ax.LMK.iImageGetPixelColor(image, line, column)
    ax.error_code(err_code) # Check for error

    return cie_r, cie_g, cie_b

def convert_cie_rgb(cie_r=255.0, cie_g=0.0, cie_b=0.0, r_ref=0.0,
                    g_ref=0.0, b_ref=0.0, color_space=dic.COLOR_SPACES['XYZ']):
    """
    Conversion of a color value from CIE-RGB to another color space.|
    ----------------------------------------------------------------
    The color value is given in CIE-RGB and is converted into a color value
    in the target color space. If there is a reference color needed for this
    color space, the reference color is also in CIE-RGB. If there is no
    reference color needed, the three variables can be set to zero.
    The destination color space is given by the value of COLOR_SPACES{}.
    The three components of the destionation color are available
    in '_Out' after the function call returned.
    -----------------------------------------------------------------------
    Parameters:
        :lmk:
            | Dispatch('lmk4.LMKAxServer')
        :cie_r: float (default: 255.0)
            | 	Red component of input color
        :cie_g: float (default: 0.0)
            | 	Green component of input color
        :cie_b: float (default: 0.0)
            | 	Blue component of input color
        :r_ref: float (default: 0.0)
            | Red component of reference color
        :g_ref: float (default: 0.0)
            | Green component of reference color
        :b_ref: float (default: 0.0)
            | Blue component of reference color
        :color_space: int (default: COLOR_SPACES['XYZ'])
            | 	Wished destination color space
    Returns:
        :output_color: array
            | Calculated color in an array shape of (1, 3)
    """
    [err_code, out_i, out_ii, out_iii] = \
        ax.LMK.iGetColor(cie_r, cie_g, cie_b, r_ref, g_ref, b_ref, color_space)
    ax.error_code(err_code) # Check for error

    # Place new values into an array
    output_color = np.array([[out_i], [out_ii], [out_iii]])
    # Transpose array into (1, 3) shape
    output_color = output_color.T

    return output_color

def xyz_to_xy(xyz):
    """
    Convert XYZ to x, y.|
    ---------------
    Parameters:
        :xyz: array (shape: (1, 3))
    Returns:
        :xy_mean: array (shape: (1, 2))
    """
    y_xy = lx.xyz_to_Yxy(xyz)
    y_xy_mean = np.array([[y_xy[:, 0].mean(), y_xy[:, 1].mean(), y_xy[:, 2].mean()]])

    y_xy_mean = np.around(y_xy_mean, decimals=3)

    x_mean = y_xy_mean[:, 1]
    x_mean = str(x_mean)[1:-1]
    y_mean = y_xy_mean[:, 2]
    y_mean = str(y_mean)[1:-1]

    xy_mean = np.array([[x_mean], [y_mean]])
    xy_mean = xy_mean.T

    return xy_mean

def xyz_to_u_v_(xyz):
    """
    Conver XYZ to u', v'.|
    ---------------
    Parameters:
        :xyz: array (shape: (1, 3))
    Returns:
        :u_v_mean: array (shape: (1, 2))
    """
    y_uv = lx.xyz_to_Yuv(xyz)
    y_uv_mean = np.array([[y_uv[:, 0].mean(), y_uv[:, 1].mean(), y_uv[:, 2].mean()]])

    y_uv_mean = np.around(y_uv_mean, decimals=3)

    u_mean = y_uv_mean[:, 1]
    u_mean = str(u_mean)[1:-1]
    v_mean = y_uv_mean[:, 2]
    v_mean = str(v_mean)[1:-1]

    u_v_mean = np.array([[u_mean], [v_mean]])
    u_v_mean = u_v_mean.T

    return u_v_mean

def show_xy(x_val, y_val, label='x, y', facecolors='none', color='k',
            title='x, y', grid=True):
    """
    Plot x, y color coordinates using Luxpy.

    Parameters:
        :x_val: float, int, or array
            | x coordinate(s)
        :y_val: float, int, or array
            | y coordinate(s)
        :label: string (default: 'x, y')
            | Change to adjust label within diagram of the input.
        :facecolors: string (default: 'none')
            | Change to adjust face color of value within diagram. Only if
                gamut=None
        :color: string (default: 'k')
            | Change to adjust color of either edge color or line color,
                depending on if 'gamut' is chosen.
        :linestyle: string (default: '--')
            | Change to adjust style of line if gamut is not None.
        :title: string (default: 'x, y')
            | Change to adjust title of figure.
        :grid: True of None (default: True)
            | Change to 'None' for no grid in diagram.
        :kwargs:
            | Additional keyword arguments for use with matplotlib.pyplot

    Returns:

    """
    plt.figure()
    ax_xy = plt.axes()
    lx.plot_chromaticity_diagram_colors(256, 0.3, 1, lx._CIEOBS, 'Yxy', {},
                                        True, ax_xy, grid, 'Times New Roman', 12)
    plt.scatter(float(x_val), float(y_val), label=label, facecolors=facecolors,
                edgecolors=color)
    ax_xy.set_title(title)
    ax_xy.set_xlim([-0.1, 0.8])
    ax_xy.set_ylim([-0.1, 0.9])
    ax_xy.legend()

def show_u_v_(u_val, v_val, label='u_, v_', facecolors='none', color='k',
              title='u_, v_', grid=True):
    """
    Plot u', v' color coordinates using Luxpy.

    Parameters:
        :u_val: float, int, or array
            | u' coordinate(s)
        :v_val: float, int, or array
            | v' coordinate(s)
        :label: string (default: 'u_, v_')
            | Change to adjust label within diagram of the input.
        :facecolors: string (default: 'none')
            | Change to adjust face color of value within diagram. Only if
                gamut=None
        :color: string (default: 'k')
            | Change to adjust color of either edge color or line color,
                depending on if 'gamut' is chosen.
        :linestyle: string (default: '--')
            | Change to adjust style of line if gamut is not None.
        :title: string (default: 'u_, v_')
            | Change to adjust title of figure.
        :grid: True of None (default: True)
            | Change to 'None' for no grid in diagram.
        :kwargs:
            | Additional keyword arguments for use with matplotlib.pyplot

    Returns:

    """
    plt.figure()
    ax_uv = plt.axes()
    lx.plot_chromaticity_diagram_colors(256, 0.3, 1, lx._CIEOBS, 'Yuv', {},
                                        True, ax_uv, grid, 'Times New Roman', 12)
    plt.scatter(float(u_val), float(v_val), label=label, facecolors=facecolors,
                edgecolors=color)
    ax_uv.set_title(title)
    ax_uv.set_xlim([-0.1, 0.7])
    ax_uv.set_ylim([-0.1, 0.7])
    ax_uv.legend()



# -*- coding: utf-8 -*-
"""
Created on Wed Mar  4 18:24:22 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)
"""
import os
import numpy as np
import matplotlib.pyplot as plt

from technopy.change_this import roots as root
from technopy.variables import dicts as dic
from technopy.technoteam import labsoft as ls
from technopy.technoteam import camera as cam
from technopy.technoteam import capture as cap
from technopy.technoteam import image as im

class Hyperspectral:
    """
    [ADD THIS]
    """

    def __init__(self, connect_camera=False):
        """
        Initializes LMK for Hyperspectral Camera.|
        -----------------------------------------
        %timeit:


        Parameters
        ----------
        None.

        Returns
        -------
        None.

        """

        self.camera = 'Hyperspectral'
        self.lens = '8mm'
        self.load_hyp = root.LOAD_HYP
        self.save_root = root.SAVE_TEST
        self.load_root = root.LOAD
        self.image = dic.IMAGE_TYPES['Luminance']
        self.file_type = dic.FILE_TYPES['pf']

        ls.open_labsoft()

        if connect_camera is True:
            _, self.camera_no = cam.set_camera(self.camera)
            _, self.lens_no = cam.set_lens(self.camera, self.lens)
            cam.open_camera(self.camera_no, self.lens_no)

    def capture(self):
        """
        [ADD THIS]

        Returns
        -------
        None.

        """

        _, filter_wheel_names = cam.get_filter_wheels()
        for number, filter_name in enumerate(filter_wheel_names[1:]):
            cam.set_filter_wheel(number+1)
            cam.set_autoscan(1)
            cap.single_pic(False)
            im.rotate(src_image=self.image, param='270', dst_image=self.image)
            im.save(self.image, self.save_root + str(filter_name)[1:-1] + \
                    dic.FILE_TYPES['txt'])


    def save_all(self):
        """
        [ADD THIS]

        """
        ## Initialize
        # Open LMK LabSoft4 Standard Color ActiveX
        ls.open_labsoft()

        for wavelength in os.listdir(self.load_hyp):
            if wavelength.endswith(self.file_type):
                self.variable = wavelength[:-3]
                im.load(self.image, self.load_hyp + wavelength)
                im.rotate(src_image=self.image, param='270', dst_image=self.image)
                im.save(self.image, self.save_root + self.variable + \
                        dic.FILE_TYPES['txt'])

    def read_all(self):
        """
        [ADD THIS]

        Returns
        -------
        wavelengths : TYPE
            DESCRIPTION.
        results : TYPE
            DESCRIPTION.

        """

        self.file_type = dic.FILE_TYPES['txt']

        self.wavelengths = []
        self.results = []

        for wavelength in os.listdir(self.load_root):
            if wavelength.endswith(self.file_type):
                variable = wavelength[:-4]
                output = np.loadtxt(self.load_root + variable + self.file_type,
                                    skiprows=2, delimiter='\t')
                self.wavelengths.append(variable)
                self.results.append(output)

        return self.wavelengths, self.results

    def get_pixel(self, row, col, data=None, nms=None, plot=False):
        """
        [ADD THIS]

        Parameters
        ----------
        row : TYPE
            DESCRIPTION.
        col : TYPE
            DESCRIPTION.
        data : TYPE, optional
            DESCRIPTION. The default is None.
        nms : TYPE, optional
            DESCRIPTION. The default is None.
        plot : TYPE, optional
            DESCRIPTION. The default is False.

        Returns
        -------
        TYPE
            DESCRIPTION.

        """

        if data and nms is not None:
            self.results = data
            self.wavelengths = nms

        self.pixel = []
        for result in self.results:
            self.max_rows = result.shape[0]
            self.max_cols = result.shape[1]
            self.pixel.append(result[row, col])

        if plot is True:
            plt.plot(self.wavelengths, self.pixel)
            plt.title('Pixel at: Row: ' + str(row) + ' Col: ' + str(col))
            plt.xlabel('Wavelength [nm]')

        return self.pixel

if __name__ == '__main__':

    HYPER = Hyperspectral(connect_camera=True)



# -*- coding: utf-8 -*-
"""
Created on Thu Mar  5 19:35:31 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)
"""
import sys
import os
import datetime
import numpy as np
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import change_this.roots as root
import variables.dicts as dic
import technoteam.labsoft as ls
import technoteam.capture as cap
import technoteam.region as reg
import technoteam.evaluation as eva
import technoteam.table as tab

def max_luminance(min_time=0.0, time_ratio=3.0, pic_count=1):
    """
    [ADD THIS]

    Parameters
    ----------
    min_time : TYPE, optional
        DESCRIPTION. The default is 0.0.
    time_ratio : TYPE, optional
        DESCRIPTION. The default is 3.0.
    pic_count : TYPE, optional
        DESCRIPTION. The default is 1.

    Returns
    -------
    max_lum : TYPE
        DESCRIPTION.

    """

    ### Capture Image
    # Capture a ColorHighDyn Image with Max Exposure Time of all Filters
    cap.color_high_dyn(autoscan=False, max_time=0.33,
                       min_time=min_time,
                       time_ratio=time_ratio, pic_count=pic_count)

    ### Evaluate
    # Create a region the size of the whole image
    _, _ = reg.create_rect_image_size()
    # Get ID of region
    _, index_out = reg.get_id(dic.IMAGE_TYPES['Luminance'], name='1')
    # Select region from index of region
    reg.select(index=index_out)

    # Create statistic
    eva.create_statistic(dic.STATISTIC_TYPES['standardGrey'],
                         dic.IMAGE_TYPES['Luminance'], index_out,
                         param_list=[1])
    # Get max luminance of region
    max_lum = tab.get_cell(table_id=2, table_line_id=0, table_column_id=6)

    max_lum = np.float64(max_lum)

    ### Save Measurement
    ls.save(file_name=root.SAVE + \
            datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S') + \
                dic.FILE_TYPES['ttcs'])

    return max_lum

def get_result_from_folder(target='XYZ', region='Center Circle'):
    """
    [ADD THIS]

    Parameters
    ----------
    target : TYPE, optional
        DESCRIPTION. The default is 'XYZ'.
    region : TYPE, optional
        DESCRIPTION. The default is 'Center Circle'.

    Returns
    -------
    results : TYPE
        DESCRIPTION.

    """

    ls.open_labsoft()

    results = []

    for measurement in os.listdir(root.LOAD):
        if measurement.endswith('.ttcs'):
            ls.load(root.LOAD + '/' + measurement)
            if target == 'XYZ':
                if region == 'Whole Image':
                    output = eva.get_image_mean_xyz()
                elif region == 'Center Circle':
                    output = eva.get_circle_mean_xyz()
            elif target == 'Y':
                if region == 'Whole Image':
                    output = max_luminance()
            results.append(output)

    if results != []:
        results = np.vstack((results))

    return results



# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='technopy',
    version='0.0.14',
    author='Gareth V. Walkom',
    author_email='walkga04@googlemail.com',
    license='GPLv3',
    description='technopy allows you to control your TechnoTeam Luminance Camera or Hyperspectral Camera.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/garethwalkom/technopy',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



# -*- coding: utf-8 -*-
"""
Created on Sun Feb  9 21:50:00 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)
"""
CAMERA_NAMES = {'Old':'ttf8847',
                'VR': 'tts20035',
                'Hyperspectral': 'tte4202ir_52127ir'}

OLD_LENSES = {'6.5': 'o13196f6_5',
              '12': 'o95653f12',
              '25': 'oB225463f25',
              '50': 'oC216813f50'}

VR_LENSES = {'50mm': 'oM00442f50',
             'Conoscopic': 'oTTC-163_D0224',
             'NED_2mm': 'oTTNED-12_50_2mmEP',
             'NED_4mm': 'oTTNED-12_50_4mmEP'}

HYPERSPECTRAL_LENSES = {'8mm':'oDietzschf8'}

## Access Color Spaces
# Dictionary containing all available color spaces.
# WARNING: All statistics are gathered in CIE-RGB. To use another color space,
# this must be converted using GetColor()
COLOR_SPACES = {
        'CIE-RGB': 1,
        'S-RGB': 2,
        'EBU-RGB': 4,
        'XYZ': 16,
        'Lxy': 32,
        'Luv': 64,
        'Lu_v_': 128,
        'L*u*v*': 256,
        'C*h*s*_uv': 512,
        'L*a*b*': 1024,
        'C*h*_ab': 2048,
        'HSV': 4096,
        'HSI': 8192,
        'WST': 16384,
        'Lrg': 32768,
        'LWS': 65536}

## Access Image
# Structure to access the images
IMAGE_TYPES = {
        'Camera': -3,
        'Luminance': -2,
        'Color': -1,
        'Evaluation[1]': 0,
        'Evaluation[2]': 1,
        'Evaluation[3]': 2,
        'Evaluation[4]': 3,
        'Evaluation[5]': 4,}

## Access Available File Types
# Not really needed, but nice to have them all in one dict
FILE_TYPES = {
        'ttcs': '.ttcs',
        'pus': '.pus',
        'pf': '.pf',
        'pcf': '.pcf',
        'txt': '.txt',
        'tix': '.tix',
        'bmp': '.bmp',
        'jpg': '.jpg',
        'png': '.png',
        'cos': '.cos',
        'csv': '.csv',
        'ini': '.ini'}

## Access Regions
# Structure to access region lists
REGION_TYPES = {
    'Rectangle': {
        'identifier': 0,
        'points': 2},
    'Line': {
        'identifier': 1,
        'points': 2},
    'Circle': {
        'identifier': 2,
        'points': 2},
    'Polygon': {
        'identifier': 3,
        'points': 3},
    'Polyline': {
        'identifier': 4,
        'points': 3},
    'Ellipse': {
        'identifier': 5,
        'points': 3},
    'CircularRing': {
        'identifier': 6,
        'points': 3},
    'OR': {
        'identifier': 7,
        'points': 2},
    'XOR': {
        'identifier': 8,
        'points':2},
    'AND': {
        'identifier': 9,
        'points': 2}}

## Access Operations
# Possible operations and their parameters
OPERATION_TYPES = {
    'Addition': 'add',
    'Subtraction': 'sub',
    'Multiplication': 'mul',
    'Division': 'div',
    'Minimum': 'min',
    'Maximum': 'max',
    'Greater than': 'grt',
    'Smaller than': 'smt',
    'Binarization': 'bin',
    'Power': 'pow',
    'Mirror': 'mir',
    'Rotate': 'rot',
    'Scaling': 'sca',
    'Box filter': 'box',
    'Median filter': 'med',
    'Erosion': 'ero',
    'Dilation': 'dil',
    'Negation': 'neg',
    'Inversion': 'inv',
    'Exponent': 'exp',
    'Logarithmus': 'log'}

## Access Statistics
# Comments show which REGION_TYPES and IMAGE_TYPES the statistic types can be used with
STATISTIC_TYPES = {
        'standardGrey': 0,
        # Rectangle, Line   :: Camera, Luminance :: Standard statistic in grey images
        'standardColor': 1,
        # Rectangle, Line   :: Color             :: Standard statistic in color images
        'sectionalGrey': 2,
        # Rectangle, Line   :: Camera, Luminance :: Sectional view in grey images
        'sectionalColor': 3,
        # Rectangle, Line   :: Color             :: Sectional view in color images
        'histogramGrey': 4,
        #                   :: [ERROR]           :: Histogram in grey images
        'histogramColor': 5,
        #                   :: [ERROR]           :: Histogram in color images
        'bitHistogramGrey': 6,
        #                   :: [ERROR]           :: Bit histogram in grey images (only
        #                                           images of camera image type)
        'bitHistorgramColor': 7,
        #                   :: [ERROR]           :: Bit histogram in color images (only
        #                                           images of color camera image type)
        'projectionGrey': 8,
        #                   :: [ERROR]           :: Projection in grey images
        'projectionColor': 9,
        #                   :: [ERROR]           :: Projection in color images
        'luminanceGrey': 20,
        #                   :: [ERROR]           :: Luminance objects in grey images
        'integralGrey': 22,
        #                   :: [ERROR]           :: Integral objects in grey images
        'integralColor': 23,
        #                   :: [ERROR]           :: Integral objects in color images
        'symbolGrey': 24,
        #                   :: [ERROR]           :: Symbol objects in grey images
        'symbolColor': 25,
        #                   :: [ERROR]           :: Symbol objects in color images
        'lightArcGrey': 26,
        #                   :: [ERROR]           :: Light arc objects in grey images
        'spiralWoundGrey': 28,
        #                   :: [ERROR]           :: Spiralwoundfilaments in grey images
        'chromaticityLineColor': 31,
        #                   :: [ERROR]           :: Chromaticity line diagrams in color
        #                                           images
        'chromaticityAreaColor': 33,
        #                   :: [ERROR]           :: Chromaticity area diagrams in color
        #                                           images
        'threeDviewGrey': 34,
        #                   :: [ERROR]           :: 3D view in grey images
        'integralNegativeGrey': 36,
        #                   :: [ERROR]           :: Integral objects in grey images
        #                                           (negative contrast)
        'integralNegativeColor': 38,
        #                   :: [EEROR]           :: Symbol objects in grey images
        #                                           (negative contrast)
        'symbolNegativeColor': 39,
        #                   :: [ERROR]           :: Symbol objects in color images
        #                                           (negative contrast)
        'contrastGrey': 40}
        #                   :: [ERROR]           :: Contrasts objects in grey images



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 19:28:01 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Structure:|
----------

create():                   Create a region.
create_rect_image_size():   Create a rectangular region the size of the whole image.
create_grid():              Create a grid region within whole image with
                                defined amount of squares.
get_id():                   Get index of region given region name.
select():                   Selects or deselects a region.
delete():                   Delete a region.

"""
import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), os.path.pardir))

import technoteam.activex as ax
import technoteam.image as im
import variables.dicts as dic

def create(image=dic.IMAGE_TYPES['Color'], region='Ellipse',
           x_coords=[1226, 500, 500], y_coords=[1026, 500, 500]):
    """
    Create a region.|
    ----------------

    Parameters
    ----------
    image : TYPE, optional
        DESCRIPTION. The default is dic.IMAGE_TYPES['Color'].
    region : TYPE, optional
        DESCRIPTION. The default is 'Ellipse'.
    x_coords : TYPE, optional
        DESCRIPTION. The default is [1226, 500, 500].
    y_coords : TYPE, optional
        DESCRIPTION. The default is [1026, 500, 500].

    Returns
    -------
    region_x_points : TYPE
        DESCRIPTION.
    region_y_points : TYPE
        DESCRIPTION.

    """
    [err_code, region_x_points, region_y_points] \
        = ax.LMK.iCreateRegion(image, dic.REGION_TYPES[region]['identifier'],
                               dic.REGION_TYPES[region]['points'], x_coords,
                               y_coords)
    ax.error_code(err_code) # Check for error

    return region_x_points, region_y_points

def create_rect(image, first_col=567, last_col=1969,
                first_line=391, last_line=2045):
    """
    [ADD THIS]

    Parameters
    ----------
    image : TYPE
        DESCRIPTION.
    first_col : TYPE, optional
        DESCRIPTION. The default is 567.
    last_col : TYPE, optional
        DESCRIPTION. The default is 1969.
    first_line : TYPE, optional
        DESCRIPTION. The default is 391.
    last_line : TYPE, optional
        DESCRIPTION. The default is 2045.

    Returns
    -------
    None.

    """

    create(image, 'Rectangle', x_coords=[first_col, last_col],
           y_coords=[first_line, last_line])

def create_rect_image_size(image=dic.IMAGE_TYPES['Color'],
                           region_id=dic.REGION_TYPES['Rectangle']['identifier'],
                           region_points=dic.REGION_TYPES['Rectangle']['points']):
    """
    Create a rectangular region the size of the whole image.|
    --------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: IMAGE_TYPES['Color'])
            | Index of region list (same as image index)
            | Im is usually defined as 'Image', but we needed to call in Image() here
        :region_id: int (default: REGION_TYPES['Rectangle']['identifier'])
            | Type of region from REGION_TYPES{}
        :NumPoints: int (default: REGION_TYPES['Rectangle']['points'])
            | Number of points from REGION_TYPES{}
    Returns:
        :region_x_points: QStringList
            | List of x-points
        :region_y_points: QStringList
            | List of y-points
    """
    [image_first_line, image_last_line, image_first_col,
     image_last_col, _] = im.get_size()
    x_coords = [image_first_col, image_last_col]
    y_coords = [image_first_line, image_last_line]
    [err_code, region_x_points, region_y_points] \
        = ax.LMK.iCreateRegion(image, region_id, region_points, x_coords, y_coords)
    ax.error_code(err_code) # Check for error

    return region_x_points, region_y_points

def create_grid(image=dic.IMAGE_TYPES['Color'],
                region_id=dic.REGION_TYPES['Rectangle']['identifier'],
                region_points=dic.REGION_TYPES['Rectangle']['points'],
                x_squares=3, y_squares=3):
    """
    Create a grid region within whole image with defined amount of squares.|
    -----------------------------------------------------------------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: IMAGE_TYPES['Color'])
            | Index of region list (same as image index)
            | Im is usually defined as 'Image', but we needed to call in Image() here
        :region_id: int (default: REGION_TYPES['Rectangle']['identifier'])
            | Type of region from REGION_TYPES{}
        :region_points: int (default: REGION_TYPES['Rectangle']['points'])
            | Number of points from REGION_TYPES{}
        :x_squares: int (default: 3)
            | Number of squares in grid along X axis
        :y_squares: int (default: 3)
            | Number of squares in grid along Y axis
    Returns:
        :X: QStringList
            | List of x-points
        :Y: QStringList
            | List of y-points
    """
    [image_first_line, image_last_line, image_first_col, image_last_col, _] = im.get_size()
    image_second_col = int(image_last_col/x_squares)
    image_third_col = int((image_last_col/x_squares)*2)
    image_second_line = int(image_last_line/y_squares)
    image_third_line = int((image_last_line/y_squares)*2)

#        X_Size = int(Image_LastColumn/X_Squares)
#        Y_Size = int(Image_LastLine/Y_Squares)
#
#        for Y in range(Image_FirstLine, Image_LastLine, Y_Size):
#            Y_Pos = ((Image_FirstColumn, Y - Image_FirstLine), \
# (Image_LastColumn, Y - Image_FirstLine))

    x_a = [image_first_col, image_second_col]
    y_a = [image_first_line, image_second_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_a, y_a)

    x_b = [image_second_col, image_third_col]
    y_b = [image_first_line, image_second_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_b, y_b)

    x_c = [image_third_col, image_last_col]
    y_c = [image_first_line, image_second_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_c, y_c)

    x_d = [image_first_col, image_second_col]
    y_d = [image_second_line, image_third_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_d, y_d)

    x_e = [image_second_col, image_third_col]
    y_e = [image_second_line, image_third_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_e, y_e)

    x_f = [image_third_col, image_last_col]
    y_f = [image_second_line, image_third_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_f, y_f)

    x_g = [image_first_col, image_second_col]
    y_g = [image_third_line, image_last_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_g, y_g)

    x_h = [image_second_col, image_third_col]
    y_h = [image_third_line, image_last_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_h, y_h)

    x_i = [image_third_col, image_last_col]
    y_i = [image_third_line, image_last_line]
    ax.LMK.iCreateRegion(image, region_id, region_points, x_i, y_i)

#        X = [Image_FirstColumn, Image_LastColumn]
#        Y = [Image_FirstLine, Image_LastLine]
#        [err_code, Region_X_Points, Region_Y_Points] = ax.LMK.iCreateRegion(Im,
# Type, NumPoints, X, Y)

def get_id(image=dic.IMAGE_TYPES['Color'], name='1'):
    """
    Get index of region given region name.|
    --------------------------------------

    %timeit:
        133 µs ± 3 µs per loop (mean ± std. dev. of 7 runs, 10000 loops each)

    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: IMAGE_TYPES['Color'])
            | Input: Index of region list (same as image index)
        :name: QString (default: '1')
            | InputL Name of region
    Returns:
        :index_out: int
            | Output: Index of this region
    """
    err_code, index_out = ax.LMK.iGetIndexOfRegion(image, name)

    return err_code, index_out

def select(image=dic.IMAGE_TYPES['Color'], index=0, check=1):
    """
    Selects or deselects a region.|
    ------------------------------

    %timeit:
        124 µs ± 376 ns per loop (mean ± std. dev. of 7 runs, 10000 loops each)

    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: IMAGE_TYPES['Color'])
            | Input: Index of region list (same as image index)
        :index: int (default: 0)
            | Index of region
        :check: int (default: 1)
            | 1= select region, 0= deselect region
    """
    err_code = ax.LMK.iSelectRegion(image, index, check)
    ax.error_code(err_code) # Check for error

def delete(image=dic.IMAGE_TYPES['Color'], index=0):
    """
    Delete a region.|
    ----------------
    Parameters:
        :LMK:
            | Dispatch('lmk4.LMKAxServer')
        :image: int (default: IMAGE_TYPES['Color'])
            | Input: Index of region list (same as image index)
        :index: int (default: 0)
            | Index of region to delete
    """
    err_code = ax.LMK.iDeleteRegion(image, index)
    ax.error_code(err_code) # Check for error
